@@ -10,8 +10,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gphoto2.h>
-#include <gphoto2-abilities-list.h>
+#include <gphoto2/gphoto2.h>
+#include <gphoto2/gphoto2-abilities-list.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
