@@ -29,9 +29,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gphoto2.h>
-#include <gphoto2-library.h>
-#include <gphoto2-result.h>
+#include <gphoto2/gphoto2.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-result.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -115,7 +115,7 @@ camera_abilities (CameraAbilitiesList *list)
 	char *ptr;
 	int   x = 0;
 
-	GP_DEBUG(" * camera_abilities()");
+	/* GP_DEBUG(" * camera_abilities()"); */
 
 	ptr = models[x].model;
 	while (ptr) {
@@ -651,6 +651,12 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 	return (GP_OK);
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_info_func = get_info_func,
+	.get_file_func = get_file_func,
+};
+
 int
 camera_init (Camera *camera, GPContext *context)
 {
@@ -666,14 +672,7 @@ camera_init (Camera *camera, GPContext *context)
         camera->functions->about                = camera_about;
 
 	/* Now, tell the filesystem where to get lists, files and info */
-	gp_filesystem_set_list_funcs (camera->fs, file_list_func,
-				      NULL, camera);
-	gp_filesystem_set_info_funcs (camera->fs, get_info_func, NULL,
-				      camera);
-	gp_filesystem_set_file_funcs (camera->fs, get_file_func,
-				      NULL, camera);
-	gp_filesystem_set_folder_funcs (camera->fs, NULL,
-					NULL, NULL, NULL, camera);
+	gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 
 	/*
 	 * The port is already provided with camera->port (and
