@@ -3,7 +3,7 @@
  *
  * Written 1999 by Wolfgang G. Reissnegger and Werner Almesberger
  *
- * $Id: canon.h 7917 2005-04-28 17:51:49Z swestin $
+ * $Id: canon.h 9396 2006-11-14 20:38:16Z marcusmeissner $
  *
  */
 
@@ -95,6 +95,7 @@ typedef enum {
  *
  */
 typedef enum {
+	CANON_CLASS_NONE,
 	CANON_CLASS_0,
 	CANON_CLASS_1,
 	CANON_CLASS_2,
@@ -130,6 +131,7 @@ typedef enum {
  * canonDownloadImageType:
  * @CANON_DOWNLOAD_THUMB: Get just the thumbnail for the image
  * @CANON_DOWNLOAD_FULL: Get the full image
+ * @CANON_DOWNLOAD_SECONDARY: Get the full secondary image
  *
  * Codes for "Download Captured Image" command to tell the camera to
  * download either the thumbnail or the full image for the most
@@ -138,7 +140,8 @@ typedef enum {
  */
 typedef enum {
 	CANON_DOWNLOAD_THUMB = 1,
-	CANON_DOWNLOAD_FULL  = 2
+	CANON_DOWNLOAD_FULL  = 2,
+	CANON_DOWNLOAD_SECONDARY  = 3
 } canonDownloadImageType;
 
 /**
@@ -166,7 +169,7 @@ typedef enum {
  */
 #define CHECK_PARAM_NULL(param) \
 	if (param == NULL) { \
-		GP_LOG (GP_LOG_ERROR, _("NULL parameter \"%s\" in %s line %i"), #param, __FILE__, __LINE__); \
+		gp_log (GP_LOG_ERROR, "canon/canon.h", _("NULL parameter \"%s\" in %s line %i"), #param, __FILE__, __LINE__); \
 		return GP_ERROR_BAD_PARAMETERS; \
 	}
 
@@ -190,6 +193,205 @@ typedef enum {
 } canonCaptureSupport;
 
 
+#ifdef CANON_EXPERIMENTAL_20D
+/**
+ * These ISO, shutter speed, aperture, etc. settings are correct for the 
+ * EOS 5D; unsure about other cameras.
+ */
+typedef enum {
+	ISO_100 = 0x48,
+	ISO_125 = 0x4b,
+	ISO_160 = 0x4d,
+	ISO_200 = 0x50,
+	ISO_250 = 0x53,
+	ISO_320 = 0x55,
+	ISO_400 = 0x58,
+	ISO_500 = 0x5b,
+	ISO_640 = 0x5d,
+	ISO_800 = 0x60,
+	ISO_1000 = 0x63,
+	ISO_1250 = 0x65,
+	ISO_1600 = 0x68
+} canonIsoState;
+
+struct canonIsoStateStruct {
+	canonIsoState value;
+	char *label;
+};
+
+typedef enum {
+	APERTURE_F2_8 = 0x20,
+	APERTURE_F3_2 = 0x23,
+	APERTURE_F3_5 = 0x25,
+	APERTURE_F4_0 = 0x28,
+	APERTURE_F4_5 = 0x2b,
+	APERTURE_F5_0 = 0x2d,
+	APERTURE_F5_6 = 0x30,
+	APERTURE_F6_3 = 0x33,
+	APERTURE_F7_1 = 0x35,
+	APERTURE_F8 = 0x38,
+	APERTURE_F9 = 0x3b,
+	APERTURE_F10 = 0x3d,
+	APERTURE_F11 = 0x40,
+	APERTURE_F13 = 0x43,
+	APERTURE_F14 = 0x45,
+	APERTURE_F16 = 0x48,
+	APERTURE_F18 = 0x4b,
+	APERTURE_F20 = 0x4d,
+	APERTURE_F22 = 0x50
+} canonApertureState;
+
+struct canonApertureStateStruct {
+	canonApertureState value;
+	char *label;
+};
+
+typedef enum {
+	SHUTTER_SPEED_BULB = 0x04,
+	SHUTTER_SPEED_30_SEC = 0x10,
+	SHUTTER_SPEED_25_SEC = 0x13,
+	SHUTTER_SPEED_20_SEC = 0x15,
+	SHUTTER_SPEED_15_SEC = 0x18,
+	SHUTTER_SPEED_13_SEC = 0x1b,
+	SHUTTER_SPEED_10_SEC = 0x1d,
+	SHUTTER_SPEED_8_SEC = 0x20,
+	SHUTTER_SPEED_6_SEC = 0x23,
+	SHUTTER_SPEED_5_SEC = 0x25,
+	SHUTTER_SPEED_4_SEC = 0x28,
+	SHUTTER_SPEED_3_2_SEC = 0x2b,
+	SHUTTER_SPEED_2_5_SEC = 0x2d,
+	SHUTTER_SPEED_2_SEC = 0x30,
+	SHUTTER_SPEED_1_6_SEC = 0x32,
+	SHUTTER_SPEED_1_3_SEC = 0x35,
+	SHUTTER_SPEED_1_SEC = 0x38,
+	SHUTTER_SPEED_0_8_SEC = 0x3b,
+	SHUTTER_SPEED_0_6_SEC = 0x3d,
+	SHUTTER_SPEED_0_5_SEC = 0x40,
+	SHUTTER_SPEED_0_4_SEC = 0x43,
+	SHUTTER_SPEED_0_3_SEC = 0x45,
+	SHUTTER_SPEED_1_4 = 0x48,
+	SHUTTER_SPEED_1_5 = 0x4b,
+	SHUTTER_SPEED_1_6 = 0x4d,
+	SHUTTER_SPEED_1_8 = 0x50,
+	SHUTTER_SPEED_1_10 = 0x53,
+	SHUTTER_SPEED_1_13 = 0x55,
+	SHUTTER_SPEED_1_15 = 0x58,
+	SHUTTER_SPEED_1_20 = 0x5b,
+	SHUTTER_SPEED_1_25 = 0x5d,
+	SHUTTER_SPEED_1_30 = 0x60,
+	SHUTTER_SPEED_1_40 = 0x63,
+	SHUTTER_SPEED_1_50 = 0x65,
+	SHUTTER_SPEED_1_60 = 0x68,
+	SHUTTER_SPEED_1_80 = 0x6b,
+	SHUTTER_SPEED_1_100 = 0x6d,
+	SHUTTER_SPEED_1_125 = 0x70,
+	SHUTTER_SPEED_1_160 = 0x73,
+	SHUTTER_SPEED_1_200 = 0x75,
+	SHUTTER_SPEED_1_250 = 0x78,
+	SHUTTER_SPEED_1_320 = 0x7b,
+	SHUTTER_SPEED_1_400 = 0x7d,
+	SHUTTER_SPEED_1_500 = 0x80,
+	SHUTTER_SPEED_1_640 = 0x83,
+	SHUTTER_SPEED_1_800 = 0x85,
+	SHUTTER_SPEED_1_1000 = 0x88,
+	SHUTTER_SPEED_1_1250 = 0x8b,
+	SHUTTER_SPEED_1_1600 = 0x8d,
+	SHUTTER_SPEED_1_2000 = 0x90,
+	SHUTTER_SPEED_1_2500 = 0x93,
+	SHUTTER_SPEED_1_3200 = 0x95,
+	SHUTTER_SPEED_1_4000 = 0x98
+} canonShutterSpeedState;
+
+struct canonShutterSpeedStateStruct {
+	canonShutterSpeedState value;
+	char *label;
+};
+
+typedef enum {
+	RESOLUTION_RAW                        = 0,
+	RESOLUTION_RAW_AND_LARGE_FINE_JPEG, 
+	RESOLUTION_RAW_AND_LARGE_NORMAL_JPEG, 
+	RESOLUTION_RAW_AND_MEDIUM_FINE_JPEG, 
+	RESOLUTION_RAW_AND_MEDIUM_NORMAL_JPEG,
+	RESOLUTION_RAW_AND_SMALL_FINE_JPEG,
+	RESOLUTION_RAW_AND_SMALL_NORMAL_JPEG,
+	RESOLUTION_LARGE_FINE_JPEG,
+	RESOLUTION_LARGE_NORMAL_JPEG,
+	RESOLUTION_MEDIUM_FINE_JPEG,
+	RESOLUTION_MEDIUM_NORMAL_JPEG,
+	RESOLUTION_SMALL_FINE_JPEG,
+	RESOLUTION_SMALL_NORMAL_JPEG
+} canonResolutionState;
+
+
+struct canonResolutionStateStruct {
+	canonResolutionState value;
+	char *label;
+	unsigned char res_byte1;
+	unsigned char res_byte2;
+	unsigned char res_byte3;
+};
+
+typedef enum {
+	AUTO_FOCUS_ONE_SHOT = 0,
+	AUTO_FOCUS_AI_SERVO,
+	AUTO_FOCUS_AI_FOCUS,
+	MANUAL_FOCUS
+} canonFocusModeState;
+
+struct canonFocusModeStateStruct {
+	canonFocusModeState value;
+	char *label;
+};
+
+
+#endif /* CANON_EXPERIMENTAL_20D */
+
+/* The following defines are for the CANON_EXPERIMENTAL_20D */
+
+/* Size of the release parameter block */
+#define RELEASE_PARAMS_LEN  0x2f
+
+/* These indexes are byte offsets into the release parameter data */
+#define RESOLUTION_1_INDEX  0x01
+#define RESOLUTION_2_INDEX  0x02
+#define RESOLUTION_3_INDEX  0x03
+#define BEEP_INDEX          0x07
+#define FOCUS_MODE_INDEX    0x12
+#define ISO_INDEX           0x1a
+#define APERTURE_INDEX      0x1c
+#define SHUTTERSPEED_INDEX  0x1e
+
+/* end of CANON_EXPERIMENTAL_20D defines */
+
+
+/**
+ * canonCaptureSizeClass:
+ * @CAPTURE_COMPATIBILITY: operate in the traditional gphoto2 mode
+ * @CAPTURE_THUMB: capture thumbnails 
+ * @CAPTURE_FULL_IMAGES: capture full-sized images
+ *
+ * By default (CAPTURE_COMPATIBILITY mode), the driver will capture
+ * thumbnails to the host computer in capture_preview, and full-sized
+ * images to the camera's drive in capture_image.  
+ * CAPTURE_FULL_IMAGE will capture a full-sized image to the host 
+ * computer in capture_preview, or to the camera's drive in capture_image.
+ * CAPTURE_THUMB is likewise intended to capture thumbnails to either 
+ * the host computer or to the camera's drive, although these modes do not
+ * seem to work right now.
+ *
+ */
+typedef enum {
+	CAPTURE_COMPATIBILITY = 1,
+	CAPTURE_THUMB,
+	CAPTURE_FULL_IMAGE
+} canonCaptureSizeClass;
+
+struct canonCaptureSizeClassStruct {
+	canonCaptureSizeClass value;
+	char *label;
+};
+
 struct canonCamModelData
 {
 	char *id_str;
@@ -235,6 +437,7 @@ struct _CameraPrivateLibrary
 	char *cached_drive;	/* usually something like C: */
 	int cached_ready;       /* whether the camera is ready to rock */
 	long image_key, thumb_length, image_length; /* For immediate download of captured image */
+	long image_b_key, image_b_length; /* For immediate download of secondary captured image */
 	int capture_step;	/* To record progress in interrupt
 				 * reads from capture */
 	int transfer_mode;	/* To remember what interrupt messages
@@ -245,6 +448,21 @@ struct _CameraPrivateLibrary
 	unsigned int xfer_length; /* Length of max transfer for
 				     download */
 
+	int remote_control;   /* is the camera currently under USB control? */
+
+	canonCaptureSizeClass capture_size; /* Size class for remote-
+                                               captured images */
+
+#ifdef CANON_EXPERIMENTAL_20D
+	unsigned char release_params[RELEASE_PARAMS_LEN]; /* "Release 
+							     parameters:"
+							     ISO, aperture, 
+							     etc */
+#endif
+
+	int secondary_image; /* Should we attempt to download a 
+				secondary image? (e.g., RAW + JPEG) */
+
 /*
  * Directory access may be rather expensive, so we cached some information.
  * This is now done by libgphoto2, so we are continuously removing this stuff.
@@ -403,18 +621,25 @@ char *canon_int_get_disk_name(Camera *camera, GPContext *context);
 int canon_int_get_battery(Camera *camera, int *pwr_status, int *pwr_source, GPContext *context);
 
 int canon_int_capture_image (Camera *camera, CameraFilePath *path, GPContext *context);
-int canon_int_capture_preview (Camera *camera, unsigned char **data, int *length,
+int canon_int_capture_preview (Camera *camera, unsigned char **data, unsigned int *length,
 			       GPContext *context);
 
 int canon_int_get_disk_name_info(Camera *camera, const char *name,int *capacity,int *available, GPContext *context);
 
 int canon_int_list_directory (Camera *camera, const char *folder, CameraList *list, const canonDirlistFunctionBits flags, GPContext *context);
 
-int canon_int_get_file(Camera *camera, const char *name, unsigned char **data, int *length, GPContext *context);
-int canon_int_get_thumbnail(Camera *camera, const char *name, unsigned char **retdata, int *length, GPContext *context);
+int canon_int_get_file(Camera *camera, const char *name, unsigned char **data, unsigned int *length, GPContext *context);
+int canon_int_get_thumbnail(Camera *camera, const char *name, unsigned char **retdata, unsigned int *length, GPContext *context);
 int canon_int_put_file(Camera *camera, CameraFile *file, char *destname, char *destpath, GPContext *context);
 int canon_int_set_file_attributes(Camera *camera, const char *file, const char *dir, canonDirentAttributeBits attrs, GPContext *context);
 int canon_int_delete_file(Camera *camera, const char *name, const char *dir, GPContext *context);
+#ifdef CANON_EXPERIMENTAL_SET_RELEASE_PARAMS
+int canon_int_set_shutter_speed(Camera *camera, canonShutterSpeedState shutter_speed, GPContext *context);
+int canon_int_set_iso(Camera *camera, canonIsoState iso, GPContext *context);
+int canon_int_set_aperture(Camera *camera, canonApertureState aperture, GPContext *context);
+int canon_int_set_focus_mode (Camera *camera, canonFocusModeState focus_mode, GPContext *context);
+int canon_int_set_resolution (Camera *camera, unsigned char res_byte1, unsigned char res_byte2, unsigned char res_byte3, GPContext *context);
+#endif
 int canon_serial_end(Camera *camera);
 int canon_serial_off(Camera *camera);
 int canon_int_get_time(Camera *camera, time_t *camera_time, GPContext *context);
@@ -422,18 +647,30 @@ int canon_int_set_time(Camera *camera, time_t date, GPContext *context);
 int canon_int_directory_operations(Camera *camera, const char *path, canonDirFunctionCode action, GPContext *context);
 int canon_int_identify_camera(Camera *camera, GPContext *context);
 int canon_int_set_owner_name(Camera *camera, const char *name, GPContext *context);
+int canon_int_start_remote_control(Camera *camera, GPContext *context);
+int canon_int_end_remote_control(Camera *camera, GPContext *context);
 
 /*
  * introduced for capturing
  */
 int
 canon_int_get_picture_abilities (Camera *camera, GPContext *context);
+#ifdef CANON_EXPERIMENTAL_20D
+int canon_int_get_release_params (Camera *camera, GPContext *context);
+#endif
+#ifdef CANON_EXPERIMENTAL_SET_RELEASE_PARAMS
+int canon_int_set_release_params(Camera *camera, GPContext *context);
+#endif
 int
-canon_int_pack_control_subcmd (unsigned char *payload, int subcmd,
+canon_int_pack_control_subcmd (unsigned char *payload, unsigned int subcmd,
 			       int word0, int word1,
 			       char *desc);
 int
-canon_int_do_control_command (Camera *camera, int subcmd, int a, int b);
+canon_int_do_control_command (Camera *camera, unsigned int subcmd, int a, int b);
+int
+canon_int_do_control_dialogue (Camera *camera, unsigned int subcmd, int a, int b, unsigned char **response_handle, unsigned int *datalen);
+int
+canon_int_do_control_dialogue_payload (Camera *camera, unsigned char *payload, unsigned int payloadlen, unsigned char **response_handle, unsigned int *datalen);
 
 
 
