@@ -4,7 +4,7 @@
  *
  * Serial communication layer.
  *
- * $Id: serial.c 7385 2004-11-17 20:28:35Z swestin $
+ * $Id: serial.c 9256 2006-10-03 19:33:16Z hun $
  *
  ****************************************************************************/
 
@@ -27,7 +27,7 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#include <gphoto2.h>
+#include <gphoto2/gphoto2.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -61,6 +61,14 @@
 # define MIN(a, b) ((a) < (b) ? (a) : (b))
 #endif
 
+
+#ifdef __GNUC__
+# define __unused__ __attribute__((unused))
+#else
+# define __unused__
+#endif
+
+
 /**
  * serial_flush_input
  * @gdev: serial port to use
@@ -69,7 +77,7 @@
  *
  */
 void
-serial_flush_input (GPPort *gdev)
+serial_flush_input (GPPort __unused__ *gdev)
 {
 }
 
@@ -81,7 +89,7 @@ serial_flush_input (GPPort *gdev)
  *
  */
 void
-serial_flush_output (GPPort *gdev)
+serial_flush_output (GPPort __unused__ *gdev)
 {
 }
 
@@ -149,7 +157,7 @@ canon_serial_init (Camera *camera)
 {
 	GPPortSettings settings;
 
-	GP_DEBUG ("Initializing the (serial) camera.\n");
+	GP_DEBUG ("Initializing the (serial) camera.");
 
 	/* Get the current settings */
 	gp_port_get_settings (camera->port, &settings);
@@ -237,7 +245,7 @@ canon_serial_get_byte (GPPort *gdev)
 		return (int) *cachep++;
 	}
 
-	recv = gp_port_read (gdev, cache, 1);
+	recv = gp_port_read (gdev, (char *)cache, 1);
 	if (recv < 0)		/* An error occurred */
 		return -1;
 
@@ -276,8 +284,9 @@ canon_serial_send_frame (Camera *camera, const unsigned char *pkt, int len)
 	p = buffer;
 	*p++ = CANON_FBEG;
 	while (len--) {
-		if (p - buffer >= sizeof (buffer) - 1) {
-			GP_DEBUG ("FATAL ERROR: send buffer overflow\n");
+		if (p < buffer ||
+		    (unsigned int)(p - buffer) >= sizeof (buffer) - 1) {
+			GP_DEBUG ("FATAL ERROR: send buffer overflow");
 			return -1;
 		}
 		if (*pkt != CANON_FBEG && *pkt != CANON_FEND && *pkt != CANON_ESC)
@@ -321,8 +330,9 @@ canon_serial_recv_frame (Camera *camera, int *len)
 			return NULL;
 		if (c == CANON_ESC)
 			c = canon_serial_get_byte (camera->port) ^ CANON_XOR;
-		if (p - buffer >= sizeof (buffer)) {
-			GP_DEBUG ("FATAL ERROR: receive buffer overflow\n");
+		if (p < buffer ||
+		    (unsigned int)(p - buffer) >= sizeof (buffer)) {
+			GP_DEBUG ("FATAL ERROR: receive buffer overflow");
 			return NULL;
 		}
 		*p++ = c;
@@ -332,7 +342,7 @@ canon_serial_recv_frame (Camera *camera, int *len)
 	 * set a lower debug level
 	 */
 	gp_log (GP_LOG_DATA, "canon", "RECV (without CANON_FBEG and CANON_FEND bytes)");
-	gp_log_data ("canon", buffer, p - buffer);
+	gp_log_data ("canon", (char *)buffer, p - buffer);
 
 	if (len)
 		*len = p - buffer;
@@ -415,14 +425,14 @@ canon_serial_recv_packet (Camera *camera, unsigned char *type, unsigned char *se
 	if (!pkt)
 		return NULL;
 	if (raw_length < PKT_HDR_LEN) {
-		GP_DEBUG ("ERROR: packet truncated\n");
+		GP_DEBUG ("ERROR: packet truncated");
 		return NULL;
 	}
 	if (pkt[PKT_TYPE] == PKT_MSG) {
 		length = pkt[PKT_LEN_LSB] | (pkt[PKT_LEN_MSB] << 8);
 		if (length + PKT_HDR_LEN > raw_length - 2) {
-			GP_DEBUG ("ERROR: invalid length\n");
-			/*fprintf(stderr,"Sending NACK\n");
+			GP_DEBUG ("ERROR: invalid length");
+			/*fprintf(stderr,"Sending NACK");
 			   canon_serial_send_packet(PKT_NACK,camera->pl->seq_rx++,camera->pl->psa50_eot+PKT_HDR_LEN,0); */
 			camera->pl->receive_error = ERROR_RECEIVED;
 			return NULL;
@@ -430,7 +440,7 @@ canon_serial_recv_packet (Camera *camera, unsigned char *type, unsigned char *se
 	}
 	crc = pkt[raw_length - 2] | (pkt[raw_length - 1] << 8);
 	if (!canon_psa50_chk_crc (pkt, raw_length - 2, crc)) {
-		GP_DEBUG ("ERROR: CRC error\n");
+		GP_DEBUG ("ERROR: CRC error");
 		return NULL;
 	}
 	if (type)
@@ -471,7 +481,7 @@ canon_serial_wait_for_ack (Camera *camera)
 			return 0;
 		if (seq == camera->pl->seq_tx && type == PKT_ACK) {
 			if (pkt[2] == PKTACK_NACK) {
-				GP_DEBUG ("ERROR: NACK received\n");
+				GP_DEBUG ("ERROR: NACK received");
 				return -1;
 			}
 			camera->pl->seq_tx++;
@@ -481,7 +491,7 @@ canon_serial_wait_for_ack (Camera *camera)
 		if (type == PKT_EOT) {
 			old_seq = pkt[0];
 			if (camera->pl->receive_error == NOERROR) {
-				GP_DEBUG ("Old EOT received, sending corresponding ACK\n");
+				GP_DEBUG ("Old EOT received, sending corresponding ACK");
 				if (!canon_serial_send_packet
 				    (camera, PKT_ACK, old_seq,
 				     camera->pl->psa50_eot + PKT_HDR_LEN, 0))
@@ -491,7 +501,7 @@ canon_serial_wait_for_ack (Camera *camera)
 					return 0;
 				if (seq == old_seq && type == PKT_ACK) {
 					if (pkt[2] == PKTACK_NACK) {
-						GP_DEBUG ("Old EOT acknowledged\n");
+						GP_DEBUG ("Old EOT acknowledged");
 						return -1;
 					}
 					return 1;
@@ -507,8 +517,8 @@ canon_serial_wait_for_ack (Camera *camera)
 			return 1;
 		}
 
-		GP_DEBUG ("ERROR: ACK format or sequence error, retrying\n");
-		GP_DEBUG ("Sending NACK\n");
+		GP_DEBUG ("ERROR: ACK format or sequence error, retrying");
+		GP_DEBUG ("Sending NACK");
 		canon_serial_send_packet (camera, PKT_NACK, camera->pl->seq_rx++,
 					  camera->pl->psa50_eot + PKT_HDR_LEN, 0);
 		camera->pl->receive_error = ERROR_RECEIVED;
@@ -553,7 +563,7 @@ canon_serial_send_msg (Camera *camera, unsigned char mtype, unsigned char dir, v
 	total = 0;
 
 	while (1) {
-		const char *str;
+		const unsigned char *str;
 		int len;
 
 		str = va_arg (*ap, unsigned char *);
@@ -563,7 +573,7 @@ canon_serial_send_msg (Camera *camera, unsigned char mtype, unsigned char dir, v
 		len = va_arg (*ap, int);
 
 		if (pos + len - pkt > MAX_MSG_SIZE && camera->pl->uploading != 1) {
-			GP_DEBUG ("FATAL ERROR: message too big (%i)\n", pos + len - pkt);
+			GP_DEBUG ("FATAL ERROR: message too big (%i)", pos + len - pkt);
 			return -1;
 		}
 		memcpy (pos, str, len);
@@ -609,11 +619,11 @@ canon_serial_send_msg (Camera *camera, unsigned char mtype, unsigned char dir, v
 				return 0;
 			good_ack = canon_serial_wait_for_ack (camera);
 			if (good_ack == -1) {
-				GP_DEBUG ("NACK received, retrying command\n");
+				GP_DEBUG ("NACK received, retrying command");
 			} else if (good_ack == 1) {
 				return good_ack;
 			} else {
-				GP_DEBUG ("No ACK received, retrying command\n");
+				GP_DEBUG ("No ACK received, retrying command");
 				if (try == 2) {
 					/* is the camera still there? */
 					if (!canon_serial_send_packet
@@ -623,7 +633,7 @@ canon_serial_send_msg (Camera *camera, unsigned char mtype, unsigned char dir, v
 					good_ack = canon_serial_wait_for_ack (camera);
 					if (good_ack == 0) {
 						camera->pl->receive_error = FATAL_ERROR;
-						GP_DEBUG ("ERROR: FATAL ERROR\n");
+						GP_DEBUG ("ERROR: FATAL ERROR");
 						clear_readiness (camera);
 						return -1;
 					}
@@ -652,7 +662,7 @@ canon_serial_send_msg (Camera *camera, unsigned char mtype, unsigned char dir, v
  *
  */
 unsigned char *
-canon_serial_recv_msg (Camera *camera, unsigned char mtype, unsigned char dir, int *total,
+canon_serial_recv_msg (Camera *camera, unsigned char mtype, unsigned char dir, unsigned int *total,
 		       GPContext *context)
 {
 	static unsigned char *msg = NULL;
@@ -670,11 +680,11 @@ canon_serial_recv_msg (Camera *camera, unsigned char mtype, unsigned char dir, i
 		/* uploading is special */
 /*              if (type == PKT_ACK && mtype == 0x3 && dir == 0x21) break; */
 		if (type == PKT_EOT) {
-			GP_DEBUG ("Old EOT received sending corresponding ACK\n");
+			GP_DEBUG ("Old EOT received sending corresponding ACK");
 			canon_serial_send_packet (camera, PKT_ACK, frag[0],
 						  camera->pl->psa50_eot + PKT_HDR_LEN, 0);
 		}
-		GP_DEBUG ("ERROR: protocol error, retrying\n");
+		GP_DEBUG ("ERROR: protocol error, retrying");
 	}
 	/* we keep the fragment only if there was no error */
 	if (camera->pl->receive_error == NOERROR) {
@@ -683,12 +693,12 @@ canon_serial_recv_msg (Camera *camera, unsigned char mtype, unsigned char dir, i
 		 * not always in the same order */
 /*
 		if (type == PKT_ACK && mtype == 0x3 && dir == 0x21) {
-			GP_DEBUG("ignoring ACK received while waiting for MSG\n");
+			GP_DEBUG("ignoring ACK received while waiting for MSG");
 			return frag;
 		} 
 */
 		if (len < MSG_HDR_LEN || frag[MSG_02] != 2) {
-			GP_DEBUG ("ERROR: message format error\n");
+			GP_DEBUG ("ERROR: message format error");
 			return NULL;
 		}
 
@@ -824,7 +834,7 @@ canon_serial_recv_msg (Camera *camera, unsigned char mtype, unsigned char dir, i
  */
 unsigned char *
 canon_serial_dialogue (Camera *camera, GPContext *context, unsigned char mtype,
-		       unsigned char dir, int *len, ...)
+		       unsigned char dir, unsigned int *len, ...)
 {
 	va_list ap;
 	int okay, try;
@@ -846,7 +856,7 @@ canon_serial_dialogue (Camera *camera, GPContext *context, unsigned char mtype,
 			if (!good_ack)
 				return NULL;
 			if (good_ack[0] == camera->pl->seq_tx && good_ack[1] == 0x5) {
-				GP_DEBUG ("ACK received waiting for the confirmation message\n");
+				GP_DEBUG ("ACK received waiting for the confirmation message");
 				good_ack =
 					canon_serial_recv_msg (camera, mtype,
 							       dir ^ DIR_REVERSE, len,
@@ -864,7 +874,7 @@ canon_serial_dialogue (Camera *camera, GPContext *context, unsigned char mtype,
 		if (good_ack)
 			return good_ack;
 		if (camera->pl->receive_error == NOERROR) {
-			GP_DEBUG ("Resending message...\n");
+			GP_DEBUG ("Resending message...");
 			camera->pl->seq_tx--;
 		}
 		if (camera->pl->receive_error == FATAL_ERROR)
@@ -888,8 +898,8 @@ canon_serial_dialogue (Camera *camera, GPContext *context, unsigned char mtype,
 int
 canon_serial_end (Camera *camera)
 {
-	canon_serial_send (camera, "\xC0\x00\x02\x55\x2C\xC1", 6, USLEEP2);
-	canon_serial_send (camera, "\xC0\x00\x04\x01\x00\x00\x00\x24\xC6\xC1", 8, USLEEP2);
+	canon_serial_send (camera, (unsigned char *)"\xC0\x00\x02\x55\x2C\xC1", 6, USLEEP2);
+	canon_serial_send (camera, (unsigned char *)"\xC0\x00\x04\x01\x00\x00\x00\x24\xC6\xC1", 8, USLEEP2);
 	return GP_OK;
 }
 
@@ -907,8 +917,8 @@ canon_serial_end (Camera *camera)
 int
 canon_serial_off (Camera *camera)
 {
-	canon_serial_send (camera, "\xC0\x00\x02\x55\x2C\xC1", 6, USLEEP2);
-	canon_serial_send (camera, "\xC0\x00\x04\x01\x00\x00\x00\x24\xC6\xC1", 8, USLEEP2);
+	canon_serial_send (camera, (unsigned char *)"\xC0\x00\x02\x55\x2C\xC1", 6, USLEEP2);
+	canon_serial_send (camera, (unsigned char *)"\xC0\x00\x04\x01\x00\x00\x00\x24\xC6\xC1", 8, USLEEP2);
 	canon_serial_change_speed (camera->port, 9600);
 	return GP_OK;
 }
@@ -928,13 +938,13 @@ canon_serial_error_type (Camera *camera)
 {
 	switch (camera->pl->receive_error) {
 		case ERROR_LOWBATT:
-			GP_DEBUG ("ERROR: no battery left, Bailing out!\n");
+			GP_DEBUG ("ERROR: no battery left, Bailing out!");
 			break;
 		case FATAL_ERROR:
-			GP_DEBUG ("ERROR: camera connection lost!\n");
+			GP_DEBUG ("ERROR: camera connection lost!");
 			break;
 		default:
-			GP_DEBUG ("ERROR: malformed message\n");
+			GP_DEBUG ("ERROR: malformed message");
 			break;
 	}
 }
@@ -963,9 +973,10 @@ canon_serial_put_file (Camera *camera, CameraFile *file, char *destname, char *d
 	char offset2[4];
 	int block_len;
 	char block_len2[4];
-	int sent = 0;
-	int i, j = 0, len, hdr_len;
-	long int size;
+	unsigned int sent = 0;
+	int i, j = 0;
+	unsigned int len, hdr_len;
+	unsigned long int size;
 	const char *data, *name;
 	unsigned int id;
 
@@ -1031,16 +1042,16 @@ canon_serial_put_file (Camera *camera, CameraFile *file, char *destname, char *d
  *
  */
 unsigned char *
-canon_serial_get_file (Camera *camera, const char *name, int *length, GPContext *context)
+canon_serial_get_file (Camera *camera, const char *name, unsigned int *length, GPContext *context)
 {
 	unsigned char *file = NULL;
 	unsigned char *msg;
 	unsigned char name_len;
 	unsigned int total = 0, expect = 0, size, id;
-	int len;
+	unsigned int len;
 
 	if (camera->pl->receive_error == FATAL_ERROR) {
-		GP_DEBUG ("ERROR: can't continue a fatal error condition detected\n");
+		GP_DEBUG ("ERROR: can't continue a fatal error condition detected");
 		return NULL;
 	}
 	name_len = strlen (name) + 1;
@@ -1059,7 +1070,7 @@ canon_serial_get_file (Camera *camera, const char *name, int *length, GPContext
 			total = le32atoh (msg + 4);
 
 			if (total > camera->pl->md->max_picture_size) {
-				GP_DEBUG ("ERROR: %d is too big\n", total);
+				GP_DEBUG ("ERROR: %d is too big", total);
 				break;
 			}
 			file = malloc (total);
@@ -1072,7 +1083,7 @@ canon_serial_get_file (Camera *camera, const char *name, int *length, GPContext
 		}
 		size = le32atoh (msg + 12);
 		if (le32atoh (msg + 8) != expect || expect + size > total || size > len - 20) {
-			GP_DEBUG ("ERROR: doesn't fit\n");
+			GP_DEBUG ("ERROR: doesn't fit");
 			break;
 		}
 		memcpy (file + expect, msg + 20, size);
@@ -1111,7 +1122,7 @@ canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 			  unsigned int *dirents_length, const char *path, GPContext *context)
 {
 	unsigned char *p, *temp_ch, *data = NULL;
-	int mallocd_bytes, total_size;
+	unsigned int mallocd_bytes, total_size;
 
 	*dirent_data = NULL;
 
@@ -1138,7 +1149,7 @@ canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 	gp_log (GP_LOG_DATA, "canon",
 		"canon_serial_get_dirents: "
 		"dirent packet received from canon_serial_dialogue:");
-	gp_log_data ("canon", p, *dirents_length);
+	gp_log_data ("canon", (char *)p, *dirents_length);
 
 	/* the first five bytes is only for the RS232 implementation
 	 * of this command, we do not need to copy them so therefore
@@ -1179,7 +1190,7 @@ canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 		gp_log (GP_LOG_DATA, "canon",
 			"canon_serial_get_dirents: "
 			"dirent packet received from canon_serial_recv_msg:");
-		gp_log_data ("canon", p, *dirents_length);
+		gp_log_data ("canon", (char *)p, *dirents_length);
 
 		/* the first five bytes is only for the RS232 implementation,
 		 * don't count them when checking dirent size
@@ -1196,9 +1207,10 @@ canon_serial_get_dirents (Camera *camera, unsigned char **dirent_data,
 		/* check if we need to allocate some more memory,
 		 * the first five bytes is only for the RS232
 		 * implementation of this command, don't need to
-		 * malloc for them.
+		 * malloc for them.  note that we ensured earlier in this
+		 * function that *dirents_length >= 5.
 		 */
-		if (total_size + (*dirents_length - 5) > mallocd_bytes) {
+		if (total_size + (unsigned int)(*dirents_length - 5) > mallocd_bytes) {
 			/* we allocate 1024 bytes chunks instead
 			 * of the exact number of bytes needed.
 			 * this is OK since we will free this
@@ -1260,7 +1272,7 @@ canon_serial_ready (Camera *camera, GPContext *context)
 {
 	unsigned char type, seq;
 	int good_ack, speed, try, len, i;
-	char *pkt;
+	unsigned char *pkt;
 	int res;
 	char cam_id_str[2000];
 	unsigned int id;
@@ -1275,7 +1287,7 @@ canon_serial_ready (Camera *camera, GPContext *context)
 
 	/* First of all, we must check if the camera is already on */
 	/*      cts=canon_serial_get_cts();
-	   GP_DEBUG("cts : %i\n",cts);
+	   GP_DEBUG("cts : %i",cts);
 	   if (cts==32) {  CTS == 32  when the camera is connected. */
 	if (camera->pl->first_init == 0 && camera->pl->cached_ready == 1) {
 		/* First case, the serial speed of the camera is the same as
@@ -1285,7 +1297,7 @@ canon_serial_ready (Camera *camera, GPContext *context)
 		     0))
 			return GP_ERROR;
 		good_ack = canon_serial_wait_for_ack (camera);
-		GP_DEBUG ("good_ack = %i\n", good_ack);
+		GP_DEBUG ("good_ack = %i", good_ack);
 		if (good_ack == 0) {
 			/* no answer from the camera, let's try
 			 * at the speed saved in the settings... */
@@ -1307,10 +1319,10 @@ canon_serial_ready (Camera *camera, GPContext *context)
 				return canon_int_ready (camera, context);
 			}
 			if (good_ack == -1) {
-				GP_DEBUG ("Received a NACK !\n");
+				GP_DEBUG ("Received a NACK!");
 				return GP_ERROR;
 			}
-			gp_context_status (context, _("Camera OK.\n"));
+			gp_context_status (context, _("Camera OK."));
 			return 1;
 		}
 		if (good_ack == -1) {
@@ -1335,7 +1347,7 @@ canon_serial_ready (Camera *camera, GPContext *context)
 	}
 	id = gp_context_progress_start (context, MAX_TRIES, _("Trying to contact camera..."));
 	for (try = 0; try < MAX_TRIES; try++) {
-		if (canon_serial_send (camera, "\x55\x55\x55\x55\x55\x55\x55\x55", 8, USLEEP1)
+		if (canon_serial_send (camera, (unsigned char *)"\x55\x55\x55\x55\x55\x55\x55\x55", 8, USLEEP1)
 		    < 0) {
 			gp_context_error (context, _("Communication error 1"));
 			return GP_ERROR;
@@ -1354,11 +1366,11 @@ canon_serial_ready (Camera *camera, GPContext *context)
 		gp_context_error (context, _("No response from camera"));
 		return GP_ERROR;
 	}
-	if (len < 40 && strncmp (pkt + 26, "Canon", 5)) {
+	if (len < 40 && strncmp ((char *)pkt + 26, "Canon", 5)) {
 		gp_context_error (context, _("Unrecognized response"));
 		return GP_ERROR;
 	}
-	strncpy (cam_id_str, pkt + 26, sizeof (cam_id_str) - 1);
+	strncpy (cam_id_str, (char *)pkt + 26, sizeof (cam_id_str) - 1);
 
 	GP_DEBUG ("cam_id_str : '%s'", cam_id_str);
 
@@ -1409,7 +1421,7 @@ canon_serial_ready (Camera *camera, GPContext *context)
 	}
 	camera->pl->seq_tx = 0;
 	camera->pl->seq_rx = 1;
-	if (!canon_serial_send_frame (camera, "\x00\x05\x00\x00\x00\x00\xdb\xd1", 8)) {
+	if (!canon_serial_send_frame (camera, (unsigned char *)"\x00\x05\x00\x00\x00\x00\xdb\xd1", 8)) {
 		gp_context_error (context, _("Communication error 2"));
 		return GP_ERROR;
 	}
@@ -1432,7 +1444,7 @@ canon_serial_ready (Camera *camera, GPContext *context)
 			break;
 	}
 
-	if (!res || !canon_serial_send_frame (camera, "\x00\x04\x01\x00\x00\x00\x24\xc6", 8)) {
+	if (!res || !canon_serial_send_frame (camera, (unsigned char *)"\x00\x04\x01\x00\x00\x00\x24\xc6", 8)) {
 		gp_context_error (context, _("Communication error 3"));
 		return GP_ERROR;
 	}
@@ -1488,7 +1500,7 @@ canon_serial_ready (Camera *camera, GPContext *context)
  */
 int
 canon_serial_get_thumbnail (Camera *camera, const char *name, unsigned char **data,
-			    int *length, GPContext *context)
+			    unsigned int *length, GPContext *context)
 {
 	unsigned int expect = 0, size, payload_length, total_file_size;
 	unsigned int total = 0, id;
