@@ -2,7 +2,7 @@
  *
  * File: usb.h
  *
- * $Id: usb.h 8062 2005-06-11 15:57:53Z hun $
+ * $Id: usb.h 8860 2006-06-12 09:53:26Z marcusmeissner $
  *
  ****************************************************************************/
 
@@ -257,20 +257,25 @@ int canon_usb_camera_init (Camera *camera, GPContext *context);
 int canon_usb_set_file_time ( Camera *camera, char *camera_filename, time_t time, GPContext *context);
 int canon_usb_put_file (Camera *camera, CameraFile *file, char *destname, char *destpath, 
 	        GPContext *context);
-unsigned char *canon_usb_capture_dialogue (Camera *camera, int *return_length, GPContext *context );
+unsigned char *canon_usb_capture_dialogue (Camera *camera, unsigned int *return_length, int *photo_status, GPContext *context );
+unsigned char *canon_usb_dialogue_full (Camera *camera, canonCommandIndex canon_funct,
+				   unsigned int *return_length, const unsigned char *payload, unsigned int payload_length);
 unsigned char *canon_usb_dialogue (Camera *camera, canonCommandIndex canon_funct,
-				   int *return_length, const char *payload, int payload_length);
+				   unsigned int *return_length, const unsigned char *payload, unsigned int payload_length);
 int canon_usb_long_dialogue (Camera *camera, canonCommandIndex canon_funct, unsigned char **data, 
-		int *data_length, int max_data_size, const char *payload,
-		int payload_length, int display_status, GPContext *context);
-int canon_usb_get_file (Camera *camera, const char *name, unsigned char **data, int *length, GPContext *context);
-int canon_usb_get_thumbnail (Camera *camera, const char *name, unsigned char **data, int *length, GPContext *context);
+		unsigned int *data_length, unsigned int max_data_size, const unsigned char *payload,
+		unsigned int payload_length, int display_status, GPContext *context);
+int canon_usb_get_file (Camera *camera, const char *name, unsigned char **data, unsigned int *length, GPContext *context);
+int canon_usb_get_thumbnail (Camera *camera, const char *name, unsigned char **data, unsigned int *length, GPContext *context);
 int canon_usb_poll_interrupt_multiple ( Camera *camera[], int n_cameras,
 					int camera_flags[],
 					unsigned char *buf, int n_tries,
 					int *which );
-int canon_usb_get_captured_image (Camera *camera, const int key, unsigned char **data, int *length, GPContext *context);
-int canon_usb_get_captured_thumbnail (Camera *camera, const int key, unsigned char **data, int *length, GPContext *context);
+int canon_usb_get_captured_image (Camera *camera, const int key, unsigned char **data, unsigned int *length, GPContext *context);
+#ifdef CANON_EXPERIMENTAL_SECONDARY_IMAGE
+int canon_usb_get_captured_secondary_image (Camera *camera, const int key, unsigned char **data, unsigned int *length, GPContext *context);
+#endif
+int canon_usb_get_captured_thumbnail (Camera *camera, const int key, unsigned char **data, unsigned int *length, GPContext *context);
 int canon_usb_lock_keys(Camera *camera, GPContext *context);
 int canon_usb_unlock_keys(Camera *camera, GPContext *context);
 int canon_usb_get_dirents (Camera *camera, unsigned char **dirent_data, unsigned int *dirents_length, const char *path, GPContext *context);
