@@ -2,13 +2,13 @@
  *
  * File: util.h
  *
- * $Id: util.h 7940 2005-05-05 01:12:46Z swestin $
+ * $Id: util.h 9330 2006-10-14 11:54:47Z hun $
  ****************************************************************************/
 
 #ifndef _CANON_UTIL_H
 #define _CANON_UTIL_H
 
-#include <gphoto2-endian.h>
+#include "gphoto2-endian.h"
 
 #ifndef TRUE
 #define TRUE 1
