@@ -3,7 +3,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gphoto2.h>
+#include <gphoto2/gphoto2.h>
 
 #include "dc120.h"
 #include "library.h"
@@ -27,7 +27,8 @@
 #  define N_(String) (String)
 #endif
 
-char *dc120_packet_new (int command_byte) {
+static char *
+dc120_packet_new (int command_byte) {
 
 	char *p = (char *)malloc(sizeof(char) * 8);
 
@@ -43,12 +44,13 @@ char *dc120_packet_new (int command_byte) {
  *  Checks if the response is ok (completed, ready, received) or not ok.
  *  @returns GP_OK if command is ok, GP_ERROR otherwise.
  */
-int dc120_response_ok (char response) {
+static int
+dc120_response_ok (unsigned char response) {
   /* This is stupid. how do you get a constant of type signed char?? (Ralf) */
-	 if( response == '\x00' || /* Command completed */
-	     response == '\x10' || /* Command ready */
-	     response == '\xd1' || /* Command received (ACK) */
-	     response == '\xd2' )  /* Correct packet */
+	 if( response == 0x00 || /* Command completed */
+	     response == 0x10 || /* Command ready */
+	     response == 0xd1 || /* Command received (ACK) */
+	     response == 0xd2 )  /* Correct packet */
 	   return (GP_OK);
          else
 	   return (GP_ERROR);
@@ -58,12 +60,13 @@ int dc120_response_ok (char response) {
  *  This function writes a packet to a port determined by camera->port.
  *  When read_response != FALSE it also reads the response.
  */
-int dc120_packet_write (Camera *camera, char *packet, int size, int read_response) {
+static int
+dc120_packet_write (Camera *camera, char *packet, int size, int read_response) {
   
 	/* Writes the packet and returns the result */
 
 	int x=0;
-	char in[2];
+	unsigned char in[2];
 
 write_again:
 	/* If retry, give camera some recup time */
@@ -96,7 +99,8 @@ write_again:
 	return (GP_OK);
 }
 
-int dc120_packet_read (Camera *camera, char *packet, int size) {
+static int
+dc120_packet_read (Camera *camera, char *packet, int size) {
 
 	return (gp_port_read(camera->port, packet, size));
 }
