@@ -27,7 +27,7 @@
 #include <bayer.h>
 #include <gamma.h>
 
-#include <gphoto2.h>
+#include <gphoto2/gphoto2.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -44,7 +44,7 @@
 #endif
 
 #include "ez200.h"
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2-port.h>
 
 #define GP_MODULE "kodak_ez200"
 
@@ -53,7 +53,7 @@ struct _CameraPrivateLibrary {
 	Info info[2];
 };
 
-struct {
+static struct {
    	char *name;
 	CameraDriverStatus status;
    	unsigned short idVendor;
