@@ -37,9 +37,9 @@
 
 #include <string.h>
 #include <stdio.h>
-#include <gphoto2-library.h>
-#include <gphoto2-result.h>
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-result.h>
+#include <gphoto2/gphoto2-port-log.h>
 #include <exif.h>
 
 #ifdef ENABLE_NLS
@@ -125,15 +125,13 @@
 #define PLAY_DATE_DISP_PTR	0x20
 #define DATE_FORMAT_PTR		0x21
 #define TIMESTAMP_PTR		0x22
-extern unsigned char k_calculate_checksum (unsigned char *buf,
-		unsigned long int len);
 
 /** Local functions **********************************************************/
 
 /* 
  * Check the integrity of datas
  */
-unsigned char
+static unsigned char
 k_calculate_checksum (unsigned char *buf, unsigned long int len)
 {
 	int i;
@@ -531,7 +529,7 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
 			gp_context_progress_stop (context, id);
 			gp_context_error(context, 
 				_("Can't upload this image to the camera. "
-				"An error has occured."));
+				"An error has occurred."));
 			return (GP_ERROR);
 		}
 		state = NEXTFRAME;
@@ -596,7 +594,7 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
 	if (ack != ACK) {
 		gp_context_progress_stop (context, id);
 		gp_context_error(context, _("Can't upload this image to the "
-			"camera. An error has occured."));
+			"camera. An error has occurred."));
 		return (GP_ERROR);
 	}
 	gp_context_progress_stop (context, id);
@@ -1206,8 +1204,8 @@ camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 {
 	snprintf(manual->text,sizeof(manual->text),
 		_("About Konica Q-M150:\n"
-		"This camera does not support to do any changes\n"
-		"from the outside. So in the configuration, you could\n"
+		"This camera does not allow any changes\n"
+		"from the outside. So in the configuration, you can\n"
 		"just see what it is configured on the camera\n"
 		"but you can't change anything.\n\n"
 		"If you have some issues with this driver, please\n"
