@@ -26,9 +26,9 @@
 #include <string.h>
 #include <math.h>
 
-#include <gphoto2.h>
-#include <gphoto2-port.h>
-#include <gphoto2-endian.h>
+#include <gphoto2/gphoto2.h>
+#include <gphoto2/gphoto2-port.h>
+#include "gphoto2-endian.h"
 
 #include "mars.h"
 
@@ -36,15 +36,24 @@
 
 #define INIT 		0xb5
 #define GET_DATA 	0x0f
-#define RESET		0xba
 
-#ifndef MAX
-# define MAX(a, b) ((a) > (b) ? (a) : (b))
-#endif
-#ifndef MIN
-# define MIN(a, b) ((a) < (b) ? (a) : (b))
-#endif
+static int 
+M_READ (GPPort *port, char *data, int size) 
+{
+	gp_port_write(port, "\x21", 1);
+    	gp_port_read(port, data, 16); 	
+	return GP_OK;
+}
 
+static int 
+M_COMMAND (GPPort *port, char *command, int size, char *response) 
+{
+	gp_port_write(port, command, size);
+    	M_READ(port, response, 16); 	
+	return GP_OK;
+}
+
+static int mars_routine (Info *info, GPPort *port, char param, int n);
 
 int 
 mars_init (Camera *camera, GPPort *port, Info *info) 
@@ -64,17 +73,15 @@ mars_init (Camera *camera, GPPort *port, Info *info)
 		gp_port_write(port, "\x19", 1);
 		gp_port_read(port, c, 16);
 	}
-	while ( (status != 0xb5)){
-	status = mars_routine (info, port, INIT, 0);    
-	GP_DEBUG("status = 0x%x\n", status);
+	else {
+		status = mars_routine (info, port, INIT, 0);    
+		GP_DEBUG("status = 0x%x\n", status);
 	}
 
-
-
 	/* Not a typo. This _will_ download the config data ;) */
 	mars_read_picture_data (camera, info, port, info, 0x2000, 0); 
 
-	/* Removing extraneous line or lines of config data. See "protocol.txt" */
+	/* Removing extraneous line(s) of data. See "protocol.txt" */
 	 
 	if ((info[0] == 0xff)&& (info[1] == 0)&&(info[2]==0xff))
 		memmove(info, info + 16, 0x1ff0); /* Saving config */
@@ -90,35 +97,16 @@ mars_get_num_pics      (Info *info)
 {
 	unsigned int i = 0;
 
-	for (i = 0; i < 0x3fe; i++) if ( !(0xff - info[8*i]) )  
-	{
-	GP_DEBUG ( "i is %i\n", i);
-	memcpy(info+0x1ff0, "i", 1);
-	return i;
-	}
+	for (i = 0; i < 0x3fe; i++) 
+		if ( !(0xff - info[8*i]) )  {
+			GP_DEBUG ( "i is %i\n", i);
+			memcpy(info+0x1ff0, "i", 1);
+			return i;
+		}
 	memcpy(info+0x1ff0, "0", 1);
 	return 0;
 }
 
-int
-mars_chk_compression (Info *info, int n)
-{
-	return	((info[8*n] >>4) & 2);
-}
-
-int
-mars_get_picture_width (Info *info, int n)
-{
-    	switch (info[8*n] & 0x0f) {  
-	case 0x00: return 176;
-	case 0x02: return 352;
-	case 0x06: return 320;
-	case 0x08: return 640;
-	default:
-		GP_DEBUG ("Your pictures have unknown width.\n");
-		return (GP_ERROR_NOT_SUPPORTED);
-	}
-}
 
 int
 mars_get_pic_data_size (Info *info, int n)
@@ -127,16 +115,18 @@ mars_get_pic_data_size (Info *info, int n)
 }
 
 int 
-set_usb_in_endpoint	     (Camera *camera, int inep) 
+set_usb_in_endpoint	(Camera *camera, int inep) 
 {
 	GPPortSettings settings;
 	gp_port_get_settings ( camera ->port, &settings);
 	settings.usb.inep = inep;
+	GP_DEBUG("inep reset to %02X\n", inep);
 	return gp_port_set_settings ( camera ->port, settings);
 }	
 
+
 int 
-mars_read_data         (Camera *camera, GPPort *port, char *data, int size) 
+mars_read_data         (GPPort *port, char *data, int size) 
 { 
 	int MAX_BULK = 0x2000;
 	int len = 0;
@@ -153,6 +143,7 @@ mars_read_data         (Camera *camera, GPPort *port, char *data, int size)
 int 
 mars_read_picture_data (Camera *camera, Info *info, GPPort *port, 
 					char *data, int size, int n) 
+
 {
 	unsigned char c[16];
 	
@@ -161,20 +152,19 @@ mars_read_picture_data (Camera *camera, Info *info, GPPort *port,
 	mars_routine (info, port, GET_DATA, n);
 	/*Data transfer begins*/
 	set_usb_in_endpoint (camera, 0x82); 
-	mars_read_data (camera, port, data, size);
-	set_usb_in_endpoint (camera, 0x83); 	
+	mars_read_data (port, data, size); 
+	set_usb_in_endpoint (camera, 0x83); 
     	return GP_OK;
 } 
 
 int
 mars_reset (GPPort *port)
 {
-	gp_port_write(port, "\x19\x54", 2);	
+	gp_port_write(port, "\x19\x54", 2);
     	return GP_OK;
 }
 
-
-void precalc_table(code_table_t *table)
+static void precalc_table(code_table_t *table)
 {
 	int i;
 	int is_abs, val, len;
@@ -184,45 +174,38 @@ void precalc_table(code_table_t *table)
 		val = 0;
 		len = 0;
 		if ((i & 0x80) == 0) {
-			/* code 0 */
-			val = 0;
-			len = 1;
-		}
-		else if ((i & 0xE0) == 0xC0) {
-			/* code 110 */
-			val = -3;
-			len = 3;
-		}
-		else if ((i & 0xE0) == 0xA0) {
-			/* code 101 */
-			val = +3;
-			len = 3;
-		}
-		else if ((i & 0xF0) == 0x80) {
-			/* code 1000 */
-			val = +8;
-			len = 4;
-		}
-		else if ((i & 0xF0) == 0x90) {
-			/* code 1001 */
-			val = -8;
-			len = 4;
-		}
-		else if ((i & 0xF0) == 0xF0) {
-			/* code 1111 */
-			val = -20;
-			len = 4;
-		}
-		else if ((i & 0xF8) == 0xE0) {
-			/* code 11100 */
-			val = +20;
-			len = 5;
-		}
-		else if ((i & 0xF8) == 0xE8) {
-			/* code 11101xxxxx */
-			is_abs = 1;
-			val = 0;	/* value is calculated later */
-			len = 5;
+    			/* code 0 */
+    			val = 0;
+    			len = 1;
+		}else if ((i & 0xE0) == 0xC0) {
+    			/* code 110 */
+    			val = -3;
+    			len = 3;
+		}else if ((i & 0xE0) == 0xA0) {
+    			/* code 101 */
+    			val = +3;
+    			len = 3;
+		}else if ((i & 0xF0) == 0x80) {
+    			/* code 1000 */
+    			val = +7;
+    			len = 4;
+		}else if ((i & 0xF0) == 0x90) {
+    			/* code 1001 */
+    			val = -7;
+    			len = 4;
+		}else if ((i & 0xF0) == 0xF0) {
+    			/* code 1111 */
+    			val = -15;
+    			len = 4;
+		}else if ((i & 0xF8) == 0xE0) {
+    			/* code 11100 */
+    			val = +15;
+    			len = 5;
+		}else if ((i & 0xF8) == 0xE8) {
+    			/* code 11101xxxxx */
+    			is_abs = 1;
+    			val = 0;	/* value is calculated later */
+    			len = 5;
 		}
 		table[i].is_abs = is_abs;
 		table[i].val = val;
@@ -230,11 +213,13 @@ void precalc_table(code_table_t *table)
 	}
 }
 
-#define CLAMP(x)	((x)<16?16:((x)>251)?251:(x))
+#define CLAMP(x)	((x)<0?0:((x)>255)?255:(x))
+#define GET_CODE 	addr = inp + (bitpos >> 3); \
+	    code = (addr[0] << (bitpos & 7)) | (addr[1] >> (8 - (bitpos & 7)))
 
 
-int mars_decompress (unsigned char *inp, 
-				unsigned char *outp, int width, int height)
+int mars_decompress (unsigned char *inp, unsigned char *outp, int width,
+   int height)
 {
 	int row, col;
 	unsigned char code;
@@ -242,10 +227,9 @@ int mars_decompress (unsigned char *inp,
 	code_table_t table[256];
 	unsigned char *addr;
 	int bitpos;
-	unsigned char A,B,C,D, predictor;
+	unsigned char lp,tp,tlp,trp;
 	/* First calculate the Huffman table */
 	precalc_table(table);
-	
 
 	bitpos = 0;
 
@@ -253,105 +237,66 @@ int mars_decompress (unsigned char *inp,
 	for (row = 0; row < height; row++) {
 		col = 0;
 
-		/* first two pixels in first two rows are stored as raw 8-bit */
-		if (row < 2) {
-			addr = inp + (bitpos >> 3);
-			code = (addr[0] << (bitpos & 7)) | (addr[1] >> (8 - (bitpos & 7)));
-			bitpos += 8;
-			*outp++ = code;
+	/* first two pixels in first two rows are stored as raw 8-bit */
+	if (row < 2) {
+    		GET_CODE;
+    		bitpos += 8;
+    		*outp++ = code;
 
-			addr = inp + (bitpos >> 3);
-			code = (addr[0] << (bitpos & 7)) | (addr[1] >> (8 - (bitpos & 7)));
-			bitpos += 8;
-			*outp++ = code;
+    		GET_CODE;
+    		bitpos += 8;
+    		*outp++ = code;
 
-			col += 2;
-		}
-
-		while (col < width) {
-			/* get bitcode */
-			addr = inp + (bitpos >> 3);
-			code = (addr[0] << (bitpos & 7)) | (addr[1] >> (8 - (bitpos & 7)));
-
-			/* update bit position */
-			bitpos += table[code].len;
-
-			/* calculate pixel value */
-			if (table[code].is_abs) {
-				/* get 5 more bits and use them as absolute value */
-				addr = inp + (bitpos >> 3);
-				code = (addr[0] << (bitpos & 7)) | (addr[1] >> (8 - (bitpos & 7)));
-				val = (code & 0xF8);
-				bitpos += 5;
-			}
-			else {
-				/* value is relative to top or left pixel */
-				val = table[code].val;
-				if (row < 2) {
-					/* top row: relative to left pixel */
-					val += outp[-2];
-				}
-				else if (col < 2) {
-					/* left column: relative to top pixel */
-					val += (outp[-2*width] + 
-						    outp[-2*width + 2])/2;
-				
-				}
-				else if (col > width - 3) {
-					/* left column: relative to top pixel */
-					val += (outp[-2*width] +
-					    outp[-2*width - 2])/2;
-				}
-				
-				else {
-					/* main area: average of left pixel and top pixel */
-					A=outp[-2];
-					C=outp[-2*width-2];
-					B=outp[-2*width ];
-					D=outp[-2*width+2];
-					if (D-A >= 0)
-						predictor= 
-						MIN((D-A)/8, 251-(A+B)/2)+ (A+B)/2;
-				
-					else 
-						predictor= 
-						-MIN((A-D)/8, (A+B)/2-4)+ (A+B)/2;
-					if ((val)&&(predictor > 0xcf))
-						val = (5*val +1)/4;
-					if ((val)&&(predictor < 0x20)) 
-						val = (5*val+3)/4;  	
-				
-					val += predictor;
-				}
-			}
-
-			/* store pixel */
-			*outp++ = CLAMP(val);
-			col++;
-		}
+    		col += 2;
 	}
 
-	return 0;
-}
+	while (col < width) {
+    		/* get bitcode */
+    		GET_CODE;
+    		/* update bit position */
+    		bitpos += table[code].len;
 
+    		/* calculate pixel value */
+    		if (table[code].is_abs) {
+    			/* get 5 more bits and use them as absolute value */
+    			GET_CODE;
+    			val = (code & 0xF8);
+    			bitpos += 5;
 
-int 
-M_READ (GPPort *port, char *data, int size) 
-{
-	gp_port_write(port, "\x21", 1);
-    	gp_port_read(port, data, 16); 	
+    			}else {
+    				/* value is relative to top or left pixel */
+    				val = table[code].val;
+    				lp =  outp[-2];
+    				if (row > 1) {
+        				tlp = outp[-2*width-2];
+        				tp  = outp[-2*width];
+        				trp = outp[-2*width+2];
+    				}
+    				if (row < 2) {
+        				/* top row: relative to left pixel */
+        				val += lp;
+    				}else if (col < 2) {
+        				/* left column: relative to top pixel */
+        				/* initial estimate */
+        				val += (2*tp + 2*trp +1)/4; 
+    				}else if (col > width - 3) {
+        				/* left column: relative to top pixel */
+        				val += (2*tp + 2*tlp +1)/4;
+					/* main area: average of left and top pixel */
+    				}else {
+        				/* initial estimate for predictor */
+					val += (2*lp + tp + trp + 1)/4;
+    				}
+    			}
+    			/* store pixel */
+    			*outp++ = CLAMP(val);
+    			col++;
+		}
+	}
 	return GP_OK;
 }
 
-int 
-M_COMMAND (GPPort *port, char *command, int size, char *response) 
-{
-	gp_port_write(port, command, size);	
-    	M_READ(port, response, 16); 	
-	return GP_OK;
-}
-
-int 
+static int 
 mars_routine (Info *info, GPPort *port, char param, int n) 
 {
 	unsigned char c[16];
@@ -368,16 +313,11 @@ mars_routine (Info *info, GPPort *port, char param, int n)
 	do_something[0]= 0x19; 
 	do_something[1]=param;
 
+
 	memset(c,0,sizeof(c));
 
 	/*Routine used in initialization, photo download, and reset. */
     	M_READ(port, c, 16); 	
-	if ( (c[0] == 0x02 ) ) {	/* Clears camera if jammed */
-		gp_port_write(port, "\x19", 1);
-		gp_port_read(port, c, 16);
-		M_READ(port, c, 16);
-	}
-
 	M_COMMAND(port, start, 2, c);
 	M_COMMAND(port, do_something, 2, c);
 	M_COMMAND(port, address1, 2, c);
@@ -393,12 +333,10 @@ mars_routine (Info *info, GPPort *port, char param, int n)
 	M_COMMAND(port, address4, 2, c);
 	M_COMMAND(port, address5, 2, c);
 	M_COMMAND(port, address6, 2, c);
-	
 	gp_port_write(port, "\x19", 1);
-	gp_port_read(port, c , 16);	
+	gp_port_read(port, c , 16);
+	/* Next thing is to switch the inep. Some cameras need a pause here */
+	usleep (MARS_SLEEP); 
 
 	return(c[0]);
 }
-
-
-
