@@ -27,7 +27,7 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <gphoto2.h>
+#include <gphoto2/gphoto2.h>
 #include <time.h>
 #include "library.h"
 #include <unistd.h>
