@@ -36,8 +36,8 @@
 #include <fcntl.h>
 #include <string.h>
 
-#include <gphoto2.h>
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2.h>
+#include <gphoto2/gphoto2-port.h>
 
 #define GP_MODULE "pccam600"
 
@@ -61,7 +61,7 @@
 #define QUALITY_ME   0x58
 #define QUALITY_HI   0x45
 
-struct models{
+static const struct models{
   char *name;
   unsigned short idVendor;
   unsigned short idProduct;
