@@ -26,12 +26,15 @@
 #define PTPBUG_DCIM_WRONG_PARENT	(1<<0)
 #define PTPBUG_DELETE_SENDS_EVENT	(1<<1)
 #define PTPBUG_DUPE_FILE		(1<<2)
+#define PTP_MTP				(1<<3)
+#define PTP_CAP				(1<<4)
+#define PTP_CAP_PREVIEW			(1<<5)
 
 #define DCIM_WRONG_PARENT_BUG(x) \
   ((x)->bugs & PTPBUG_DCIM_WRONG_PARENT)
 #define DELETE_SENDS_EVENT(x) \
   ((x)->bugs & PTPBUG_DELETE_SENDS_EVENT)
 #define CAN_HAVE_DUPE_FILE(x) \
-  ((x)->bugs & PTPBUG_DUPE_FILE)
+  ((x)->bugs & (PTPBUG_DUPE_FILE | PTP_MTP))
 
 #endif
