@@ -1,5 +1,7 @@
 /* currently this file is included into ptp.c */
 
+#include <iconv.h>
+
 static inline uint16_t
 htod16p (PTPParams *params, uint16_t var)
 {
@@ -88,32 +90,108 @@ static inline char*
 ptp_unpack_string(PTPParams *params, unsigned char* data, uint16_t offset, uint8_t *len)
 {
 	int i;
-	char *string=NULL;
+	uint8_t loclen;
 
-	*len=dtoh8a(&data[offset]);
-	if (*len) {
-		string=malloc(*len);
-		memset(string, 0, *len);
-		for (i=0;i<*len && i< PTP_MAXSTRLEN; i++) {
-			string[i]=(char)dtoh16a(&data[offset+i*2+1]);
+	/* Cannot exceed 255 (PTP_MAXSTRLEN) since it is a single byte, duh ... */
+	loclen = dtoh8a(&data[offset]);
+	/* This len is used to advance the buffer pointer */
+	*len = loclen;
+	if (loclen) {
+		uint16_t string[PTP_MAXSTRLEN+1];
+		char *stringp = (char *) string;
+		char loclstr[PTP_MAXSTRLEN*3+1]; /* UTF-8 encoding is max 3 bytes per UCS2 char. */
+		char *locp = loclstr;
+		size_t nconv;
+		size_t convlen = loclen * 2; /* UCS-2 is 16 bit wide */
+		size_t convmax = PTP_MAXSTRLEN*3;
+		
+		for (i=0;i<loclen;i++) {
+			string[i]=dtoh16a(&data[offset+i*2+1]);
 		}
-		/* be paranoid! :( */
-		string[*len-1]=0;
+		/* be paranoid! Add a terminator. :( */
+		string[loclen]=0x0000U;
+		loclstr[0]='\0';
+		/* loclstr=ucs2_to_utf8(string); */
+		/* Do the conversion.  */
+		nconv = iconv (params->cd_ucs2_to_locale, &stringp, &convlen, &locp, &convmax);
+		/* FIXME: handle size errors */
+		loclstr[PTP_MAXSTRLEN*3] = '\0';
+		if (nconv == (size_t) -1)
+			return NULL;
+		return strdup(loclstr);
 	}
-	return (string);
+	return NULL;
 }
 
+
+static inline int
+ucs2strlen(uint16_t const * const unicstr)
+{
+	int length;
+	
+	/* Unicode strings are terminated with 2 * 0x00 */
+	for(length = 0; unicstr[length] != 0x0000U; length ++);
+	return length;
+}
+
+
 static inline void
 ptp_pack_string(PTPParams *params, char *string, unsigned char* data, uint16_t offset, uint8_t *len)
 {
 	int i;
-	*len = (uint8_t)strlen(string);
-	
-	/* XXX: check strlen! */
-	htod8a(&data[offset],*len+1);
-	for (i=0;i<*len && i< PTP_MAXSTRLEN; i++) {
-		htod16a(&data[offset+i*2+1],(uint16_t)string[i]);
+	int packedlen;
+	uint16_t ucs2str[PTP_MAXSTRLEN+1];
+	char *ucs2strp = (char *) ucs2str;
+	char *stringp = string;
+	size_t nconv;
+	size_t convlen = strlen(string);
+	size_t convmax = PTP_MAXSTRLEN * 2; /* Includes the terminator */
+
+	/* Cannot exceed 255 (PTP_MAXSTRLEN) since it is a single byte, duh ... */
+	ucs2str[0] = 0x0000U;
+	memset(ucs2strp, 0, PTP_MAXSTRLEN*2+2);
+	nconv = iconv (params->cd_locale_to_ucs2, &stringp, &convlen, &ucs2strp, &convmax);
+	if (nconv == (size_t) -1)
+		ucs2str[0] = 0x0000U;
+	packedlen = ucs2strlen(ucs2str);
+	if (packedlen > PTP_MAXSTRLEN-1) {
+		*len=0;
+		return;
 	}
+	
+	/* number of characters + leading 0 */
+	htod8a(&data[offset],packedlen+1);
+	for (i=0;i<packedlen && i< PTP_MAXSTRLEN; i++) {
+		htod16a(&data[offset+i*2+1],ucs2str[i]);
+	}
+
+	/* The returned length is in number of characters */
+	*len = (uint8_t) packedlen;
+}
+
+static inline unsigned char *
+ptp_get_packed_stringcopy(PTPParams *params, char *string, uint32_t *packed_size)
+{
+	uint8_t packed[PTP_MAXSTRLEN*2+3], len;
+	size_t plen;
+	unsigned char *retcopy = NULL;
+
+	ptp_pack_string(params, string, (unsigned char*) packed, 0, &len);
+	/* returned length is in characters, then one byte for string length */
+	plen = len*2 + 1;
+	/* Assure proper termination, two zero bytes */
+	packed[plen] = 0x00;
+	packed[plen+1] = 0x00;
+	/* Include terminator */
+	plen += 2;
+	retcopy = malloc(plen);
+	if (!retcopy) {
+		*packed_size = 0;
+		return NULL;
+	}
+	memcpy(retcopy, packed, plen);
+	*packed_size = plen;
+	return (retcopy);
 }
 
 static inline uint32_t
@@ -130,6 +208,18 @@ ptp_unpack_uint32_t_array(PTPParams *params, unsigned char* data, uint16_t offse
 	return n;
 }
 
+static inline uint32_t
+ptp_pack_uint32_t_array(PTPParams *params, uint32_t *array, uint32_t arraylen, unsigned char **data )
+{
+	uint32_t i=0;
+
+	*data = malloc ((arraylen+1)*sizeof(uint32_t));
+	htod32a(&(*data)[0],arraylen);
+	for (i=0;i<arraylen;i++)
+		htod32a(&(*data)[sizeof(uint32_t)*(i+1)], array[i]);
+	return (arraylen+1)*sizeof(uint32_t);
+}
+
 static inline uint32_t
 ptp_unpack_uint16_t_array(PTPParams *params, unsigned char* data, uint16_t offset, uint16_t **array)
 {
@@ -159,7 +249,7 @@ ptp_unpack_DI (PTPParams *params, unsigned char* data, PTPDeviceInfo *di, unsign
 	uint8_t len;
 	unsigned int totallen;
 	
-	di->StaqndardVersion = dtoh16a(&data[PTP_di_StandardVersion]);
+	di->StandardVersion = dtoh16a(&data[PTP_di_StandardVersion]);
 	di->VendorExtensionID =
 		dtoh32a(&data[PTP_di_VendorExtensionID]);
 	di->VendorExtensionVersion =
@@ -734,12 +824,7 @@ ptp_pack_DPV (PTPParams *params, PTPPropertyValue* value, unsigned char** dpvptr
 		break;
 	/* XXX: other int types are unimplemented */
 	case PTP_DTC_STR: {
-		uint8_t len;
-
-		size=strlen(value->str)*2+3;
-		dpv=malloc(size);
-		memset(dpv,0,size);
-		ptp_pack_string(params, value->str, dpv, 0, &len);
+		dpv=ptp_get_packed_stringcopy(params, value->str, &size);
 		break;
 	}
 	}
@@ -747,6 +832,104 @@ ptp_pack_DPV (PTPParams *params, PTPPropertyValue* value, unsigned char** dpvptr
 	return size;
 }
 
+#define MAX_MTP_PROPS 127
+static inline uint32_t
+ptp_pack_OPL (PTPParams *params, MTPPropList *proplist, unsigned char** opldataptr, uint32_t objectid)
+{
+	unsigned char* opldata;
+	MTPPropList *propitr;
+	unsigned char *packedprops[MAX_MTP_PROPS];
+	uint32_t packedpropslens[MAX_MTP_PROPS];
+	uint16_t packedpropsids[MAX_MTP_PROPS];
+	uint16_t packedpropstypes[MAX_MTP_PROPS];
+	uint32_t totalsize = 0;
+	uint32_t bufp = 0;
+	uint32_t noitems = 0;
+	uint32_t i;
+
+	totalsize = sizeof(uint32_t); /* 4 bytes to store the number of elements */
+	propitr = proplist;
+	while (propitr != NULL && noitems < MAX_MTP_PROPS) {
+		/* Overhead for each item */
+		totalsize += sizeof(uint32_t); /* Object ID */
+		/* Metadata type */
+		packedpropsids[noitems]=propitr->property;
+		totalsize += sizeof(uint16_t);
+		/* Data type */
+		packedpropstypes[noitems]= propitr->datatype;
+		totalsize += sizeof(uint16_t);
+		/* Add each property to be sent. */
+	        packedpropslens[noitems] = ptp_pack_DPV (params, &propitr->propval, &packedprops[noitems], propitr->datatype);
+		totalsize += packedpropslens[noitems];
+		noitems ++;
+		propitr = propitr->next;
+	}
+
+	/* Allocate memory for the packed property list */
+	opldata = malloc(totalsize);
+
+	htod32a(&opldata[bufp],noitems);
+	bufp += 4;
+
+	/* Copy into a nice packed list */
+	for (i = 0; i < noitems; i++) {
+		/* Object ID */
+		htod32a(&opldata[bufp],objectid);
+		bufp += sizeof(uint32_t);
+		htod16a(&opldata[bufp],packedpropsids[i]);
+		bufp += sizeof(uint16_t);
+		htod16a(&opldata[bufp],packedpropstypes[i]);
+		bufp += sizeof(uint16_t);
+		/* The copy the actual property */
+		memcpy(&opldata[bufp], packedprops[i], packedpropslens[i]);
+		bufp += packedpropslens[i];
+		free(packedprops[i]);
+	}
+	*opldataptr = opldata;
+	return totalsize;
+}
+
+static inline int
+ptp_unpack_OPL (PTPParams *params, unsigned char* data, MTPPropList **proplist, unsigned int len)
+{ 
+	uint32_t prop_count = dtoh32a(data);
+	MTPPropList *prop = NULL;
+	int offset = 0, i;
+
+	if (prop_count == 0) {
+		*proplist = NULL;
+		return 0;
+	}
+	data += sizeof(uint32_t);
+	*proplist = malloc(sizeof(MTPPropList));
+	prop = *proplist;
+	for (i = 0; i < prop_count; i++) {
+		/* we ignore the object handle */
+		data += sizeof(uint32_t);
+		len -= sizeof(uint32_t);
+
+		prop->property = dtoh32a(data);
+		data += sizeof(uint16_t);
+		len -= sizeof(uint16_t);
+
+		prop->datatype = dtoh32a(data);
+		data += sizeof(uint16_t);
+		len -= sizeof(uint16_t);
+
+		offset = 0;
+		ptp_unpack_DPV(params, data, &offset, len, &prop->propval, prop->datatype);
+		data += offset;
+		len -= offset;
+
+		if (i != prop_count - 1) {
+			prop->next = malloc(sizeof(MTPPropList));
+			prop = prop->next;
+		} else
+			prop->next = NULL;
+	}
+	return prop_count;
+}
+
 /*
     PTP USB Event container unpack
     Copyright (c) 2003 Nikolai Kopanygin
@@ -880,3 +1063,140 @@ ptp_pack_EK_text(PTPParams *params, PTPEKTextParams *text, unsigned char **data)
 	}
 	return len;
 }
+
+#define ptp_canon_dir_version	0x00
+#define ptp_canon_dir_ofc	0x02
+#define ptp_canon_dir_unk1	0x04
+#define ptp_canon_dir_objectid	0x08
+#define ptp_canon_dir_parentid	0x0c
+#define ptp_canon_dir_previd	0x10	/* in same dir */
+#define ptp_canon_dir_nextid	0x14	/* in same dir */
+#define ptp_canon_dir_nextchild	0x18	/* down one dir */
+#define ptp_canon_dir_storageid	0x1c	/* only in storage entry */
+#define ptp_canon_dir_name	0x20
+#define ptp_canon_dir_flags	0x2c
+#define ptp_canon_dir_size	0x30
+#define ptp_canon_dir_unixtime	0x34
+#define ptp_canon_dir_year	0x38
+#define ptp_canon_dir_month	0x39
+#define ptp_canon_dir_mday	0x3a
+#define ptp_canon_dir_hour	0x3b
+#define ptp_canon_dir_minute	0x3c
+#define ptp_canon_dir_second	0x3d
+#define ptp_canon_dir_unk2	0x3e
+#define ptp_canon_dir_thumbsize	0x40
+#define ptp_canon_dir_width	0x44
+#define ptp_canon_dir_height	0x48
+
+static inline uint16_t
+ptp_unpack_canon_directory (
+	PTPParams		*params,
+	unsigned char		*dir,
+	uint32_t		cnt,
+	PTPObjectHandles	*handles,
+	PTPObjectInfo		**oinfos,	/* size(handles->n) */
+	uint32_t		**flags		/* size(handles->n) */
+) {
+	unsigned int	i, j, nrofobs = 0, curob = 0;
+
+#define ISOBJECT(ptr) (dtoh32a((ptr)+ptp_canon_dir_storageid) == 0xffffffff)
+	for (i=0;i<cnt;i++)
+		if (ISOBJECT(dir+i*0x4c)) nrofobs++;
+	handles->n = nrofobs;
+	handles->Handler = calloc(sizeof(handles->Handler[0]),nrofobs);
+	if (!handles->Handler) return PTP_RC_GeneralError;
+	*oinfos = calloc(sizeof((*oinfos)[0]),nrofobs);
+	if (!*oinfos) return PTP_RC_GeneralError;
+	*flags  = calloc(sizeof((*flags)[0]),nrofobs);
+	if (!*flags) return PTP_RC_GeneralError;
+
+	/* Migrate data into objects ids, handles into
+	 * the object handler array.
+	 */
+	curob = 0;
+	for (i=0;i<cnt;i++) {
+		unsigned char	*cur = dir+i*0x4c;
+		PTPObjectInfo	*oi = (*oinfos)+curob;
+
+		if (!ISOBJECT(cur))
+			continue;
+
+		handles->Handler[curob] = dtoh32a(cur + ptp_canon_dir_objectid);
+		oi->StorageID		= 0xffffffff;
+		oi->ObjectFormat	= dtoh16a(cur + ptp_canon_dir_ofc);
+		oi->ParentObject	= dtoh32a(cur + ptp_canon_dir_parentid);
+		oi->Filename		= strdup((char*)(cur + ptp_canon_dir_name));
+		oi->ObjectCompressedSize= dtoh32a(cur + ptp_canon_dir_size);
+		oi->ThumbCompressedSize	= dtoh32a(cur + ptp_canon_dir_thumbsize);
+		oi->ImagePixWidth	= dtoh32a(cur + ptp_canon_dir_width);
+		oi->ImagePixHeight	= dtoh32a(cur + ptp_canon_dir_height);
+		oi->CaptureDate		= oi->ModificationDate = dtoh32a(cur + ptp_canon_dir_unixtime);
+		(*flags)[curob]		= dtoh32a(cur + ptp_canon_dir_flags);
+		curob++;
+	}
+	/* Walk over Storage ID entries and distribute the IDs to
+	 * the parent objects. */
+	for (i=0;i<cnt;i++) {
+		unsigned char	*cur = dir+i*0x4c;
+		uint32_t	nextchild = dtoh32a(cur + ptp_canon_dir_nextchild);
+
+		if (ISOBJECT(cur))
+			continue;
+		for (j=0;j<handles->n;j++) if (nextchild == handles->Handler[j]) break;
+		if (j == handles->n) continue;
+		(*oinfos)[j].StorageID = dtoh32a(cur + ptp_canon_dir_storageid);
+	}
+	/* Walk over all objects and distribute the storage ids */
+	while (1) {
+		int changed = 0;
+		for (i=0;i<cnt;i++) {
+			unsigned char	*cur = dir+i*0x4c;
+			uint32_t	oid = dtoh32a(cur + ptp_canon_dir_objectid);
+			uint32_t	nextoid = dtoh32a(cur + ptp_canon_dir_nextid);
+			uint32_t	nextchild = dtoh32a(cur + ptp_canon_dir_nextchild);
+			uint32_t	storageid;
+
+			if (!ISOBJECT(cur))
+				continue;
+			for (j=0;j<handles->n;j++) if (oid == handles->Handler[j]) break;
+			if (j == handles->n) {
+				/*fprintf(stderr,"did not find oid in lookup pass for current oid\n");*/
+				continue;
+			}
+	 		storageid = (*oinfos)[j].StorageID;
+			if (storageid == 0xffffffff) continue;
+			if (nextoid != 0xffffffff) {
+				for (j=0;j<handles->n;j++) if (nextoid == handles->Handler[j]) break;
+				if (j == handles->n) {
+					/*fprintf(stderr,"did not find oid in lookup pass for next oid\n");*/
+					continue;
+				}
+				if ((*oinfos)[j].StorageID == 0xffffffff) {
+					(*oinfos)[j].StorageID = storageid;
+					changed++;
+				}
+			}
+			if (nextchild != 0xffffffff) {
+				for (j=0;j<handles->n;j++) if (nextchild == handles->Handler[j]) break;
+				if (j == handles->n) {
+					/*fprintf(stderr,"did not find oid in lookup pass for next child\n");*/
+					continue;
+				}
+				if ((*oinfos)[j].StorageID == 0xffffffff) {
+					(*oinfos)[j].StorageID = storageid;
+					changed++;
+				}
+			}
+		}
+		/* Check if we:
+		 * - changed no entry (nothing more to do)
+		 * - changed all of them at once (usually happens)
+		 * break if we do.
+		 */
+		if (!changed || (changed==nrofobs-1))
+			break;
+	}
+#undef ISOBJECT
+	return PTP_RC_OK;
+}
+
