@@ -23,6 +23,8 @@
  * - Some configure checking magic missing for the special header files 
  *   and functions.
  * - Not everything implementation correctly cross checked.
+ * - Coolpix P3 does not give transfer status (image 000x/000y), and reports an
+ *   error when transfers finish correctly.
  */
 #include "config.h"
 
@@ -31,14 +33,15 @@
 #include <stdio.h>
 #include <time.h>
 #include <errno.h>
+#include <sys/types.h>
 #include <sys/select.h>
 
 #include <sys/socket.h>
 #include <netinet/in.h>
 
-#include <gphoto2-library.h>
-#include <gphoto2-port-log.h>
-#include <gphoto2-setting.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-port-log.h>
+#include <gphoto2/gphoto2-setting.h>
 
 
 #ifdef ENABLE_NLS
@@ -207,12 +210,17 @@ ptp_ptpip_check_event (PTPParams* params) {
 #define WRITE_BLOCKSIZE 65536
 uint16_t
 ptp_ptpip_senddata (PTPParams* params, PTPContainer* ptp,
-			unsigned char *data, unsigned int size)
+			unsigned char *data, unsigned int size,
+			int to_fd)
 {
 	unsigned char	request[16];
 	int		ret, curwrite, towrite;
 	unsigned char*	xdata;
 
+	if (to_fd != -1) {
+		gp_log (GP_LOG_ERROR, "ptpip/senddata", "fd transfers not supported.");
+		return PTP_RC_GeneralError;
+	}
 	htod32a(&request[ptpip_type],PTPIP_START_DATA_PACKET);
 	htod32a(&request[ptpip_len],sizeof(request));
 	htod32a(&request[ptpip_startdata_transid  + 8],ptp->Transaction_ID);
@@ -262,13 +270,18 @@ ptp_ptpip_senddata (PTPParams* params, PTPContainer* ptp,
 
 uint16_t
 ptp_ptpip_getdata (PTPParams* params, PTPContainer* ptp,
-		unsigned char **data, unsigned int *readlen)
+		unsigned char **data, unsigned int *readlen, int to_fd)
 {
 	PTPIPHeader		hdr;
 	unsigned char		*xdata = NULL;
 	uint16_t 		ret;
 	int			toread, curread;
 
+	if (to_fd != -1) {
+		gp_log (GP_LOG_ERROR, "ptpip/getdata", "fd transfers not handled yet");
+		return PTP_RC_GeneralError;
+	}
+
 	ret = ptp_ptpip_cmd_read (params, &hdr, &xdata);
 	if (ret != PTP_RC_OK)
 		return ret;
@@ -374,11 +387,10 @@ ptp_ptpip_init_command_request (PTPParams* params)
 	char		hostname[100];
 	unsigned char*	cmdrequest;
 	int 		len,i,ret;
-	unsigned char	guid[] = {
-		0x61,0x10,0x85,0x7e, 0xfa,0xae,0x98,0x49,
-		0x8c,0xa1,0x5e,0x7d, 0x8f,0x9d,0x81,0x29
-	};
-
+	unsigned char	guid[16];
+	
+	ptp_nikon_getptpipguid(guid);
+	
 	if (gethostname (hostname, sizeof(hostname)))
 		return PTP_RC_GeneralError;
 	len = ptpip_initcmd_name + (strlen(hostname)+1)*2 + 4;
@@ -507,6 +519,66 @@ ptp_ptpip_event_wait (PTPParams* params, PTPContainer* event) {
 	return ptp_ptpip_event (params, event, PTP_EVENT_CHECK);
 }
 
+/**
+ * ptp_nikon_getwifiguid:
+ *
+ * This command gets the GUID of this machine. If it does not exists, it creates
+ * one.
+ *  
+ * params:	PTPParams*
+ *
+ * Return values: Some PTP_RC_* code.
+ *
+ **/
+void
+ptp_nikon_getptpipguid (unsigned char* guid) {
+	char buffer[1024];
+	int i;
+	long val;
+	int valid;
+	char* endptr;
+	char* pos;
+	
+	gp_setting_get("ptp2_ip","guid",buffer);
+	
+	if (strlen(buffer) == 47) { /* 47 = 16*2 (numbers) + 15 (semi-colons) */
+		pos = buffer;
+		valid = 1;
+		for (i = 0; i < 16; i++) {
+			val = strtol(pos, &endptr, 16);
+			if (((*endptr != ':') && (*endptr != 0)) || (endptr != pos+2)) {
+				valid = 0;
+				break;
+			}
+			guid[i] = (unsigned char)val;
+			pos += 3;
+		}
+		/*printf("GUID ");
+		for (i = 0; i < 16; i++) {
+			printf("%02x:", guid[i]);
+		}
+		printf("\n");*/
+		if (valid)
+			return;
+	}
+	
+	/*fprintf(stderr, "Invalid GUID\n");*/
+	
+	/* Generate an ID */
+	srand(time(NULL));
+	buffer[0] = 0;
+	pos = buffer;
+	for (i = 0; i < 16; i++) {
+		guid[i] = (unsigned char) ((256.0 * rand()) / RAND_MAX);
+		pos += sprintf(pos, "%02x:", guid[i]);
+	}
+	buffer[47] = 0;
+	
+	/*printf("New GUID: %s\n", buffer);*/
+	
+	gp_setting_set("ptp2_ip","guid",buffer);
+}
+
 
 int
 ptp_ptpip_connect (PTPParams* params, const char *address) {
