@@ -25,9 +25,9 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <gphoto2-library.h>
-#include <gphoto2-result.h>
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-result.h>
+#include <gphoto2/gphoto2-port-log.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -301,6 +301,11 @@ camera_abilities (CameraAbilitiesList *list)
 	a.usb_product	= 0x2216;
 	gp_abilities_list_append(list, a);
 
+	strcpy(a.model, "Ricoh:Caplio R4");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x2217;
+	gp_abilities_list_append(list, a);
+
 	return (GP_OK);
 }
 
