@@ -26,7 +26,7 @@
 #include <sys/types.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gphoto2-library.h>
+#include <gphoto2/gphoto2-library.h>
 #include "sierra.h"
 #include "library.h"
 #include "sierra-desc.h"
