@@ -23,7 +23,7 @@
 #endif
 #include <netinet/in.h>
 
-#include <gphoto2.h>
+#include <gphoto2/gphoto2.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -41,7 +41,7 @@
 
 #include "soundvision.h"
 
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2-port.h>
 
 #define GP_MODULE "soundvision"
 
