@@ -30,8 +30,8 @@
 #include <stdio.h>
 #include <string.h>
 
-#include <gphoto2.h>
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2.h>
+#include <gphoto2/gphoto2-port.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -78,7 +78,7 @@ static int cam_has_flash (CameraPrivateLibrary *pl);
 static int cam_has_card (CameraPrivateLibrary *pl);
 
 /* define what cameras we support */
-static struct
+static const struct
 {
 	char *model;
 	int usb_vendor;
@@ -513,7 +513,7 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 
 		gp_filesystem_name (fs, "/", c - 1, &name, context);
 		gp_context_error (context,
-				  _("Your camera does only support deleting the "
+				  _("Your camera only supports deleting the "
 				   "last file on the camera. In this case, this "
 				   "is file '%s'."), name);
 		return (GP_ERROR);
