@@ -23,8 +23,8 @@
 #include <stdio.h>
 #include <string.h>
 
-#include <gphoto2.h>
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2.h>
+#include <gphoto2/gphoto2-port.h>
 
 #include "stv0680.h"
 #include "library.h"
