@@ -1,7 +1,7 @@
 /** \file gphoto2-abilities-list.c
  * \brief List of supported camera models including their abilities.
  *
- * \author Copyright  2000 Scott Fritzinger
+ * \author Copyright 2000 Scott Fritzinger
  *
  * \par
  * This library is free software; you can redistribute it and/or
@@ -23,7 +23,7 @@
  */
 
 #include "config.h"
-#include "gphoto2-abilities-list.h"
+#include <gphoto2/gphoto2-abilities-list.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -31,9 +31,9 @@
 
 #include <ltdl.h>
 
-#include "gphoto2-result.h"
-#include "gphoto2-port-log.h"
-#include "gphoto2-library.h"
+#include <gphoto2/gphoto2-result.h>
+#include <gphoto2/gphoto2-port-log.h>
+#include <gphoto2/gphoto2-library.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -65,15 +65,6 @@
 /** \internal */
 #define CHECK_MEM(m)         {if (!(m)) return (GP_ERROR_NO_MEMORY);}
 
-/**
- * Name of the environment variable which may contain the path where
- * to look for the camlibs. If this environment variable is not defined,
- * use the compiled-in default constant.
- *
- * \internal Internal use only.
- */
-#define CAMLIBDIR_ENV "CAMLIBS"
-
 /** \internal */
 struct _CameraAbilitiesList {
 	int count;
@@ -144,17 +135,23 @@ gp_abilities_list_free (CameraAbilitiesList *list)
 }
 
 
+typedef struct {
+	CameraList *list;
+	int result;
+} foreach_data_t;
+
+
 static int
 foreach_func (const char *filename, lt_ptr data)
 {
-	CameraList *list = data;
-	int result;
+	foreach_data_t *fd = data;
+	CameraList *list = fd->list;
 
 	gp_log (GP_LOG_DEBUG, "gphoto2-abilities-list",
 		"Found '%s'.", filename);
-	result = gp_list_append (list, filename, NULL);
+	fd->result = gp_list_append (list, filename, NULL);
 
-	return ((result == GP_OK)?0:1);
+	return ((fd->result == GP_OK)?0:1);
 }
 
 
@@ -170,7 +167,6 @@ gp_abilities_list_load_dir (CameraAbilitiesList *list, const char *dir,
 	const char *filename;
 	CameraList *flist;
 	int count;
-	int ret;
 	lt_dlhandle lh;
 
 	CHECK_NULL (list && dir);
@@ -179,17 +175,22 @@ gp_abilities_list_load_dir (CameraAbilitiesList *list, const char *dir,
 		"Using ltdl to load camera libraries from '%s'...", dir);
 	CHECK_RESULT (gp_list_new (&flist));
 	CHECK_RESULT (gp_list_reset (flist));
-	lt_dlinit ();
-	lt_dladdsearchdir (dir);
-	ret = lt_dlforeachfile (dir, foreach_func, flist);
-	lt_dlexit ();
-	if (ret != 0) {
-		gp_log (GP_LOG_ERROR, "gp-abilities-list", 
-			"Internal error looking for camlibs (%d)", ret);
-		gp_context_error (context,
-				  _("Internal error looking for camlibs. "
-				    "(path names too long?)"));
-		return (GP_ERROR);
+	if (1) { /* a new block in which we can define a temporary variable */
+		int ret;
+		foreach_data_t foreach_data = { flist, GP_OK };
+		lt_dlinit ();
+		lt_dladdsearchdir (dir);
+		ret = lt_dlforeachfile (dir, foreach_func, &foreach_data);
+		lt_dlexit ();
+		if (ret != 0) {
+			gp_list_free (flist);
+			gp_log (GP_LOG_ERROR, "gp-abilities-list", 
+				"Internal error looking for camlibs (%d)", ret);
+			gp_context_error (context,
+					  _("Internal error looking for camlibs. "
+					    "(path names too long?)"));
+			return (foreach_data.result!=GP_OK)?foreach_data.result:GP_ERROR;
+		}
 	}
 	CHECK_RESULT (count = gp_list_count (flist));
 	gp_log (GP_LOG_DEBUG, "gp-abilities-list", "Found %i "
@@ -319,6 +320,9 @@ gp_abilities_list_detect_usb (CameraAbilitiesList *list,
 	for (i = 0; i < count; i++) {
 		int v, p, c, s;
 
+		if (!(list->abilities[i].port & GP_PORT_USB))
+			continue;
+
 		v = list->abilities[i].usb_vendor;
 		p = list->abilities[i].usb_product;
 		if (v) {
@@ -330,7 +334,7 @@ gp_abilities_list_detect_usb (CameraAbilitiesList *list,
 					v, p);
 				*ability = i;
 			} else if (res < 0 && res != GP_ERROR_IO_USB_FIND) {
-				/* another error occured. 
+				/* another error occurred. 
 				 * perhaps we should better
 				 * report this to the calling
 				 * method?
@@ -357,7 +361,7 @@ gp_abilities_list_detect_usb (CameraAbilitiesList *list,
 					c, s, p);
 				*ability = i;
 			} else if (res < 0 && res != GP_ERROR_IO_USB_FIND) {
-				/* another error occured. 
+				/* another error occurred. 
 				 * perhaps we should better
 				 * report this to the calling
 				 * method?
@@ -644,6 +648,59 @@ gp_abilities_list_get_abilities (CameraAbilitiesList *list, int index,
 }
 
 
+#ifdef _GPHOTO2_INTERNAL_CODE
+
+/* enum CameraOperation */
+const StringFlagItem gpi_camera_operation_map[] = {
+	{ "none",            GP_OPERATION_NONE },
+	{ "capture_image",   GP_OPERATION_CAPTURE_IMAGE },
+	{ "capture_video",   GP_OPERATION_CAPTURE_VIDEO },
+	{ "capture_audio",   GP_OPERATION_CAPTURE_AUDIO },
+	{ "capture_preview", GP_OPERATION_CAPTURE_PREVIEW },
+	{ "config",          GP_OPERATION_CONFIG },
+	{ NULL, 0 },
+};
+
+/* enum CameraFileOperation */
+const StringFlagItem gpi_file_operation_map[] = {
+	{ "none",    GP_FILE_OPERATION_NONE },
+	{ "delete",  GP_FILE_OPERATION_DELETE },
+	{ "preview", GP_FILE_OPERATION_PREVIEW },
+	{ "raw",     GP_FILE_OPERATION_RAW },
+	{ "audio",   GP_FILE_OPERATION_AUDIO },
+	{ "exif",    GP_FILE_OPERATION_EXIF },
+	{ NULL, 0 },
+};
+
+/* enum CameraFolderOperation */
+const StringFlagItem gpi_folder_operation_map[] = {
+	{ "none",       GP_FOLDER_OPERATION_NONE },
+	{ "delete_all", GP_FOLDER_OPERATION_DELETE_ALL },
+	{ "put_file",   GP_FOLDER_OPERATION_PUT_FILE },
+	{ "make_dir",   GP_FOLDER_OPERATION_MAKE_DIR },
+	{ "remove_dir", GP_FOLDER_OPERATION_REMOVE_DIR },
+	{ NULL, 0 },
+};
+
+/* enum GphotoDeviceType */
+const StringFlagItem gpi_gphoto_device_type_map[] = {
+	{ "still_camera", GP_DEVICE_STILL_CAMERA },
+	{ "audio_player", GP_DEVICE_AUDIO_PLAYER },
+	{ NULL, 0 },
+};
+
+/* enum CameraDriverStatus */
+const StringFlagItem gpi_camera_driver_status_map[] = {
+	{ "production",   GP_DRIVER_STATUS_PRODUCTION },
+	{ "testing",      GP_DRIVER_STATUS_TESTING },
+	{ "experimental", GP_DRIVER_STATUS_EXPERIMENTAL },
+	{ "deprecated",   GP_DRIVER_STATUS_DEPRECATED },
+	{ NULL, 0 },
+};
+
+#endif /* _GPHOTO2_INTERNAL_CODE */
+
+
 /*
  * Local Variables:
  * c-file-style:"linux"
