@@ -1,6 +1,6 @@
 /* gphoto2-filesys.c
  *
- * Copyright  2000 Scott Fritzinger
+ * Copyright 2000 Scott Fritzinger
  *
  * Contributions:
  *      Lutz Mller <lutz@users.sf.net> (2001)
@@ -22,17 +22,17 @@
  */
 
 #include "config.h"
-#include "gphoto2-filesys.h"
+#include <gphoto2/gphoto2-filesys.h>
 
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <unistd.h>
 
-#include "gphoto2-result.h"
-#include "gphoto2-file.h"
-#include "gphoto2-port-log.h"
-#include "gphoto2-setting.h"
+#include <gphoto2/gphoto2-result.h>
+#include <gphoto2/gphoto2-file.h>
+#include <gphoto2/gphoto2-port-log.h>
+#include <gphoto2/gphoto2-setting.h>
 
 #include <limits.h>
 
@@ -385,6 +385,14 @@ delete_all_folders (CameraFilesystem *fs, const char *folder,
                         if (strlen (fs->folder[x].name) <= strlen (folder))
                                 continue;
 
+			/* Handle "/foo/bar" "/foo/bar bar" style directories,
+			 * where on is contained in the other, but not a subdirectory
+			 */
+			if (	(fs->folder[x].name[strlen(folder)] != '/') &&
+				(fs->folder[x].name[strlen(folder)] != '\0')
+			)
+				continue;
+
                         CR (delete_all_files (fs, x));
                         CR (delete_folder (fs, x));
                         x--;
@@ -413,9 +421,9 @@ append_folder (CameraFilesystem *fs, const char *folder, GPContext *context)
 		if (!strcmp (fs->folder[x].name, folder))
 			break;
 	if (x < fs->count) {
-		gp_context_error (context, _("Could not append folder '%s' "
-			"as this folder already exists."), folder);
-                return (GP_ERROR_DIRECTORY_EXISTS);
+		gp_log (GP_LOG_DEBUG, "filesys", "Folder %s "
+			"is duplicated, return OK.", folder);
+                return (GP_OK);
         }
 
         /* Make sure the parent exist. If not, create it. */
@@ -961,8 +969,8 @@ gp_filesystem_list_folders (CameraFilesystem *fs, const char *folder,
 			memset (buf, 0, sizeof (buf));
 			strncpy (buf, folder, MIN (sizeof (buf), len));
 			if (buf[strlen (buf) - 1] != '/')
-				strncat (buf, "/", sizeof (buf));
-			strncat (buf, name, sizeof (buf));
+				strncat (buf, "/", sizeof (buf) - strlen (buf) - 1);
+			strncat (buf, name, sizeof (buf) - strlen (buf) - 1);
 			CR (append_folder (fs, buf, context));
 		}
 		gp_list_reset (list);
@@ -978,6 +986,13 @@ gp_filesystem_list_folders (CameraFilesystem *fs, const char *folder,
 			if (strlen (fs->folder[x].name) <= len)
 				continue;
 
+			/* handle "/Music/foo " "/Music/foo bar" style directories */
+			if (	(len > 1) &&	/* ignore for / directory, see len calculation above. */
+				(fs->folder[x].name[len] != '/') &&
+				(fs->folder[x].name[len] != '\0')
+			)
+				continue;
+
 			/*
 			 * Is this really a direct subfolder (and not a 
 			 * subsubfolder)?
@@ -1066,6 +1081,8 @@ gp_filesystem_delete_file (CameraFilesystem *fs, const char *folder,
 	CR (x = gp_filesystem_folder_number (fs, folder, context));
 	CR (y = gp_filesystem_number (fs, folder, filename, context));
 
+	gp_context_status (context, _("Deleting '%s' from folder '%s'..."),
+			   filename, folder);
 	/* Delete the file */
 	CR (fs->delete_file_func (fs, folder, filename,
 				  fs->file_data, context));
@@ -1122,8 +1139,8 @@ gp_filesystem_make_dir (CameraFilesystem *fs, const char *folder,
 
 	strncpy (path, folder, sizeof (path));
 	if (path[strlen (path) - 1] != '/')
-		strncat (path, "/", sizeof (path));
-	strncat (path, name, sizeof (path));
+		strncat (path, "/", sizeof (path) - strlen (path) - 1);
+	strncat (path, name, sizeof (path) - strlen (path) - 1);
 
 	/* Create the directory */
 	CR (fs->make_dir_func (fs, folder, name, fs->folder_data, context));
@@ -1155,8 +1172,8 @@ gp_filesystem_remove_dir (CameraFilesystem *fs, const char *folder,
 	 */
 	strncpy (path, folder, sizeof (path));
 	if (path[strlen (path) - 1] != '/')
-		strncat (path, "/", sizeof (path));
-	strncat (path, name, sizeof (path));
+		strncat (path, "/", sizeof (path) - strlen (path) - 1);
+	strncat (path, name, sizeof (path) - strlen (path) - 1);
 	CL (gp_filesystem_list_folders (fs, path, list, context), list);
 	if (gp_list_count (list)) {
 		gp_context_error (context, _("There are still subfolders in "
@@ -1339,8 +1356,8 @@ gp_filesystem_scan (CameraFilesystem *fs, const char *folder,
 		CL (gp_list_get_name (list, x, &name), list);
 		strncpy (path, folder, sizeof (path));
 		if (path[strlen (path) - 1] != '/')
-			strncat (path, "/", sizeof (path));
-		strncat (path, name, sizeof (path));
+			strncat (path, "/", sizeof (path) - strlen (path) - 1);
+		strncat (path, name, sizeof (path) - strlen (path) - 1);
 		CL (gp_filesystem_scan (fs, path, filename, context), list);
 	}
 	gp_list_free (list);
