@@ -1,6 +1,6 @@
 /* gphoto2-library.c: Dummy file for gtk-doc
  *
- * Copyright  2001 Lutz Mller <lutz@users.sf.net>
+ * Copyright 2001 Lutz Mller <lutz@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,7 +19,7 @@
  */
 
 #include "config.h"
-#include "gphoto2-library.c"
+#include <gphoto2/gphoto2-library.c>
 
 /**
  * camera_id:
