@@ -1,6 +1,6 @@
 /* gphoto2-result.c
  *
- * Copyright  2000 Scott Fritzinger
+ * Copyright 2000 Scott Fritzinger
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,7 +19,7 @@
  */
 
 #include "config.h"
-#include "gphoto2-result.h"
+#include <gphoto2/gphoto2-result.h>
 
 #include <stddef.h>
 
