@@ -19,7 +19,7 @@
  */
 
 #include "config.h"
-#include "gphoto2-port-info-list.h"
+#include <gphoto2/gphoto2-port-info-list.h>
 
 #include <stdlib.h>
 #include <string.h>
@@ -32,9 +32,9 @@
 
 #include <ltdl.h>
 
-#include <gphoto2-port-result.h>
-#include <gphoto2-port-library.h>
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-port-result.h>
+#include <gphoto2/gphoto2-port-library.h>
+#include <gphoto2/gphoto2-port-log.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -51,6 +51,7 @@
 #  define dgettext(Domain,Message) (Message)
 #  define dcgettext(Domain,Message,Type) (Message)
 #  define bindtextdomain(Domain,Directory) (Domain)
+#  define bind_textdomain_codeset(Domain,codeset) (codeset)
 #  define _(String) (String)
 #  define N_(String) (String)
 #endif
@@ -462,3 +463,15 @@ gp_port_info_list_get_info (GPPortInfoList *list, int n, GPPortInfo *info)
 
 	return (GP_OK);
 }
+
+
+#ifdef _GPHOTO2_INTERNAL_CODE
+const StringFlagItem gpi_gphoto_port_type_map[] = {
+	{ "none",   GP_PORT_NONE },
+	{ "serial", GP_PORT_SERIAL },
+	{ "usb",    GP_PORT_USB },
+	{ "disk",   GP_PORT_DISK },
+	{ "ptpip",  GP_PORT_PTPIP },
+	{ NULL, 0 },
+};
+#endif /* _GPHOTO2_INTERNAL_CODE */
