@@ -20,6 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+
 #define GP_USB_HOTPLUG_SCRIPT "usbcam"
 
 /* Not sure whether this is the best possible name */
@@ -44,6 +45,7 @@ ARGV0 " prints the camera list in the specified format FORMAT on stdout.\n" \
 
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <sys/time.h>
 #include <time.h>
 #include <string.h>
@@ -58,18 +60,22 @@ ARGV0 " prints the camera list in the specified format FORMAT on stdout.\n" \
 #define FALSE (0!=0)
 #endif
 
+typedef char *string_array_t[];
+
+typedef string_array_t *string_array_p;
 
 typedef struct {
 	int number_of_cameras;
 	int add_comments;
 	int argc;
-	char ***argv;
+	string_array_p argv;
 } func_params_t;
 
 
 typedef int (* begin_func_t)  (const func_params_t *params);
 typedef int (* camera_func_t) (const func_params_t *params, 
 			       const int i,
+			       const int total,
 			       const CameraAbilities *ca);
 typedef int (* end_func_t)    (const func_params_t *params);
 
@@ -105,6 +111,13 @@ typedef int (* end_func_t)    (const func_params_t *params);
 	} while (0)
 #endif /* __GNUC__ */
 
+#define FATAL(msg...) \
+	do { \
+		fprintf(stderr, ARGV0 ": Fatal: " msg); \
+		fprintf(stderr, "\n"); \
+		exit(13); \
+	} while (0)
+
 
 /* print_usb_usermap
  *
@@ -118,38 +131,42 @@ typedef int (* end_func_t)    (const func_params_t *params);
 static int
 hotplug_camera_func (const func_params_t *params, 
 		     const int i,
+		     const int total,
 		     const CameraAbilities *a)
 {
 	int flags = 0;
 	int class = 0, subclass = 0, proto = 0;
 	int usb_vendor = 0, usb_product = 0;
 	const char *usermap_script = 
-		(*(params->argv)[0] != NULL)
-		?(*(params->argv)[0])
+		((*params->argv)[0] != NULL)
+		?((*params->argv)[0])
 		:(GP_USB_HOTPLUG_SCRIPT);
 
-	if (a->usb_vendor) { /* usb product id may be zero! */
-		class = 0;
-		subclass = 0;
-		proto = 0;
-		flags = GP_USB_HOTPLUG_MATCH_VENDOR_ID | GP_USB_HOTPLUG_MATCH_PRODUCT_ID;
-		usb_vendor = a->usb_vendor;
-		usb_product = a->usb_product;
-	} else if (a->usb_class) {
-		class = a->usb_class;
-		subclass = a->usb_subclass;
-		proto = a->usb_protocol;
-		flags = GP_USB_HOTPLUG_MATCH_INT_CLASS;
-		if (subclass != -1)
-			flags |= GP_USB_HOTPLUG_MATCH_INT_SUBCLASS;
-		else
+	if (a->port & GP_PORT_USB) {
+		if (a->usb_vendor) { /* usb product id may be zero! */
+			class = 0;
 			subclass = 0;
-		if (proto != -1)
-			flags |= GP_USB_HOTPLUG_MATCH_INT_PROTOCOL;
-		else
 			proto = 0;
-		usb_vendor = 0;
-		usb_product = 0;
+			flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID 
+				 | GP_USB_HOTPLUG_MATCH_PRODUCT_ID);
+			usb_vendor = a->usb_vendor;
+			usb_product = a->usb_product;
+		} else if ((a->usb_class) && (a->usb_class != 666)) {
+			class = a->usb_class;
+			subclass = a->usb_subclass;
+			proto = a->usb_protocol;
+			flags = GP_USB_HOTPLUG_MATCH_INT_CLASS;
+			if (subclass != -1)
+				flags |= GP_USB_HOTPLUG_MATCH_INT_SUBCLASS;
+			else
+				subclass = 0;
+			if (proto != -1)
+				flags |= GP_USB_HOTPLUG_MATCH_INT_PROTOCOL;
+			else
+				proto = 0;
+			usb_vendor = 0;
+			usb_product = 0;
+		}
 	} else {
 		/* not a USB camera */
 		return 0;
@@ -233,6 +250,7 @@ basename (const char *pathname)
 static int
 human_camera_func (const func_params_t *params, 
 		   const int i, 
+		   const int total,
 		   const CameraAbilities *a)
 {
 	const char *camlib_basename;
@@ -249,6 +267,7 @@ human_camera_func (const func_params_t *params,
 static int
 idlist_camera_func (const func_params_t *params, 
 		   const int i, 
+		   const int total,
 		   const CameraAbilities *a)
 {
 	if (a->usb_vendor) { /* usb product id may be zero! */
@@ -264,8 +283,16 @@ idlist_camera_func (const func_params_t *params,
 static int
 udev_begin_func (const func_params_t *params)
 {
-	printf ("# udev rules file for libgphoto2\n#\n");
-	printf ("BUS!=\"usb\", ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n\n");	
+	printf ("# udev rules file for libgphoto2 devices (udev < 0.98)\n#\n");
+	printf ("BUS!=\"usb\", ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n\n");
+	return 0;
+}
+
+static int
+udev_098_begin_func (const func_params_t *params)
+{
+	printf ("# udev rules file for libgphoto2 devices (udev >= 0.98)\n#\n");
+	printf ("SUBSYSTEMS!=\"usb\", ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n\n");
 	return 0;
 }
 
@@ -282,63 +309,219 @@ udev_end_func (const func_params_t *params)
 static int
 udev_camera_func (const func_params_t *params, 
 		  const int i,
+		  const int total,
 		  const CameraAbilities *a)
 {
 	int flags = 0;
 	int class = 0, subclass = 0, proto = 0;
 	int usb_vendor = 0, usb_product = 0;
-	const char *hotplug_script =
-		(*(params->argv)[0] != NULL)
-		?(*(params->argv)[0])
-		:("/etc/hotplug/usb/" GP_USB_HOTPLUG_SCRIPT);
 
-	if (a->usb_vendor) { /* usb product id may be zero! */
-		class = 0;
-		subclass = 0;
-		proto = 0;
-		flags = GP_USB_HOTPLUG_MATCH_VENDOR_ID | GP_USB_HOTPLUG_MATCH_PRODUCT_ID;
-		usb_vendor = a->usb_vendor;
-		usb_product = a->usb_product;
-	} else if (a->usb_class) {
-		class = a->usb_class;
-		subclass = a->usb_subclass;
-		proto = a->usb_protocol;
-		flags = GP_USB_HOTPLUG_MATCH_INT_CLASS;
-		if (subclass != -1)
-			flags |= GP_USB_HOTPLUG_MATCH_INT_SUBCLASS;
-		else
+	if (a->port & GP_PORT_USB) {
+		if (a->usb_vendor) { /* usb product id may be zero! */
+			class = 0;
 			subclass = 0;
-		if (proto != -1)
-			flags |= GP_USB_HOTPLUG_MATCH_INT_PROTOCOL;
-		else
 			proto = 0;
-		usb_vendor = 0;
-		usb_product = 0;
+			flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID 
+				 | GP_USB_HOTPLUG_MATCH_PRODUCT_ID);
+			usb_vendor = a->usb_vendor;
+			usb_product = a->usb_product;
+		} else if ((a->usb_class) && (a->usb_class != 666)) {
+			class = a->usb_class;
+			subclass = a->usb_subclass;
+			proto = a->usb_protocol;
+			flags = GP_USB_HOTPLUG_MATCH_INT_CLASS;
+			if (subclass != -1)
+				flags |= GP_USB_HOTPLUG_MATCH_INT_SUBCLASS;
+			else
+				subclass = 0;
+			if (proto != -1)
+				flags |= GP_USB_HOTPLUG_MATCH_INT_PROTOCOL;
+			else
+				proto = 0;
+			usb_vendor = 0;
+			usb_product = 0;
+		}
 	} else {
-		/* not a USB camera */
+		/* not a camera handled by udev */
 		return 0;
 	}
 
 	if (params->add_comments) {
 		printf ("# %s\n", a->model);
 	}
-	
+
 	if (flags & GP_USB_HOTPLUG_MATCH_INT_CLASS) {
-		printf("SYSFS{bInterfaceClass}==\"%02x\", ", class);
-		if (flags & GP_USB_HOTPLUG_MATCH_INT_SUBCLASS) {
-			printf("SYSFS{bInterfaceSubClass}==\"%02x\", ", subclass);
-		}
-		if (flags & GP_USB_HOTPLUG_MATCH_INT_PROTOCOL) {
-			printf("SYSFS{bInterfaceProtocol}==\"%02x\", ", proto);
-			}
+		if ((flags & (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) == (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL))
+			printf("ENV{INTERFACE}==\"%d/%d/%d\", ", class, subclass, proto);
+		else
+			fprintf(stderr,"unhandled interface match flags %x\n", flags);
 	} else {
 		printf ("SYSFS{idVendor}==\"%04x\", SYSFS{idProduct}==\"%04x\", ",
 			a->usb_vendor, a->usb_product);
 	}
-	printf("RUN+=\"%s\"\n", hotplug_script);
+	if ((*params->argv)[1] == NULL) {
+		const char *hotplug_script = ((*params->argv)[0] != NULL)
+			?((*params->argv)[0])
+			:("/etc/hotplug/usb/" GP_USB_HOTPLUG_SCRIPT);
+		printf("RUN+=\"%s\"\n", hotplug_script);
+	} else { /* more than two parameters in params->argv */
+		char *mode = NULL;
+		char *owner = NULL;
+		char *group = NULL;
+
+		int i;
+		char *key = NULL, *val = NULL;
+		for (i=0; ((key=(*params->argv)[i])   != NULL) && 
+			  ((val=(*params->argv)[i+1]) != NULL); i+=2) {
+			if (strcmp("owner", key)==0) {
+				owner = val;
+			} else if (strcmp("group", key)==0) {
+				group = val;
+			} else if (strcmp("mode", key)==0) {
+				mode = val;
+			} else {
+				FATAL("Unknown key argument: %s", key);
+			}
+		}
+		if ((key != NULL) && (val == NULL)) {
+			FATAL("Single argument remaining; need pairs of key and value");
+		}
+
+		if (mode != NULL || owner != NULL || group != NULL) {
+			if (mode != NULL) {
+				printf("MODE=\"%s\"", mode);
+				if (owner != NULL || group != NULL) {
+					printf(", ");
+				}
+			}
+			if (owner != NULL) {
+				printf("OWNER=\"%s\"", owner);
+				if (group != NULL) {
+					printf(", ");
+				}
+			}
+			if (group != NULL) {
+				printf("GROUP=\"%s\"", group);
+			}
+			printf("\n");
+		} else {
+			FATAL("Arguments required");
+		}
+	}
+	/*
+	printf("MUH!\n");
+	*/
 	return 0;
 }
 
+static int
+udev_098_camera_func (const func_params_t *params, 
+		  const int i,
+		  const int total,
+		  const CameraAbilities *a)
+{
+	int flags = 0;
+	int class = 0, subclass = 0, proto = 0;
+	int usb_vendor = 0, usb_product = 0;
+
+	if (a->port & GP_PORT_USB) {
+		if (a->usb_vendor) { /* usb product id may be zero! */
+			class = 0;
+			subclass = 0;
+			proto = 0;
+			flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID 
+				 | GP_USB_HOTPLUG_MATCH_PRODUCT_ID);
+			usb_vendor = a->usb_vendor;
+			usb_product = a->usb_product;
+		} else if ((a->usb_class) && (a->usb_class != 666)) {
+			class = a->usb_class;
+			subclass = a->usb_subclass;
+			proto = a->usb_protocol;
+			flags = GP_USB_HOTPLUG_MATCH_INT_CLASS;
+			if (subclass != -1)
+				flags |= GP_USB_HOTPLUG_MATCH_INT_SUBCLASS;
+			else
+				subclass = 0;
+			if (proto != -1)
+				flags |= GP_USB_HOTPLUG_MATCH_INT_PROTOCOL;
+			else
+				proto = 0;
+			usb_vendor = 0;
+			usb_product = 0;
+		}
+	} else {
+		/* not a camera handled by udev */
+		return 0;
+	}
+
+	if (params->add_comments)
+		printf ("# %s\n", a->model);
+
+	if (flags & GP_USB_HOTPLUG_MATCH_INT_CLASS) {
+		if ((flags & (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) == (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL))
+			printf("ENV{INTERFACE}==\"%d/%d/%d\", ", class, subclass, proto);
+		else
+			fprintf(stderr,"unhandled interface match flags %x\n", flags);
+	} else {
+		printf ("ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ",
+			a->usb_vendor, a->usb_product);
+	}
+	if ((*params->argv)[1] == NULL) {
+		const char *hotplug_script = ((*params->argv)[0] != NULL)
+			?((*params->argv)[0])
+			:("/etc/hotplug/usb/" GP_USB_HOTPLUG_SCRIPT);
+		printf("RUN+=\"%s\"\n", hotplug_script);
+	} else { /* more than two parameters in params->argv */
+		char *mode = NULL;
+		char *owner = NULL;
+		char *group = NULL;
+
+		int i;
+		char *key = NULL, *val = NULL;
+		for (i=0; ((key=(*params->argv)[i])   != NULL) && 
+			  ((val=(*params->argv)[i+1]) != NULL); i+=2) {
+			if (strcmp("owner", key)==0) {
+				owner = val;
+			} else if (strcmp("group", key)==0) {
+				group = val;
+			} else if (strcmp("mode", key)==0) {
+				mode = val;
+			} else {
+				FATAL("Unknown key argument: %s", key);
+			}
+		}
+		if ((key != NULL) && (val == NULL)) {
+			FATAL("Single argument remaining; need pairs of key and value");
+		}
+
+		if (mode != NULL || owner != NULL || group != NULL) {
+			if (mode != NULL) {
+				printf("MODE=\"%s\"", mode);
+				if (owner != NULL || group != NULL) {
+					printf(", ");
+				}
+			}
+			if (owner != NULL) {
+				printf("OWNER=\"%s\"", owner);
+				if (group != NULL) {
+					printf(", ");
+				}
+			}
+			if (group != NULL) {
+				printf("GROUP=\"%s\"", group);
+			}
+			printf("\n");
+		} else {
+			FATAL("Arguments required");
+		}
+	}
+	/*
+	printf("MUH!\n");
+	*/
+	return 0;
+}
+
+
 
 static int
 empty_begin_func (const func_params_t *params)
@@ -353,6 +536,173 @@ empty_end_func (const func_params_t *params)
 }
 
 
+
+static int
+ddb_begin_func (const func_params_t *params)
+{
+	printf("# Beginning of gphoto2 device database (PRE-ALPHA format!)\n\n");
+	return 0;
+}
+
+
+static int
+ddb_end_func (const func_params_t *params)
+{
+	printf("# End of gphoto2 device database (PRE-ALPHA format!).\n");
+	return 0;
+}
+
+
+static void
+ddb_list_out_func(const char *str, void *data)
+{
+	int *first = (int *)data;
+	printf("%s %s", (*first)?"":",", str);
+	*first = 0;
+}
+
+
+static void
+ddb_delayed_head(
+	const func_params_t *params, 
+	const int i,
+	const int total,
+	const CameraAbilities *a,
+	const char *camlib_basename)
+{
+	int first = 1;
+	printf("# This is a PRE-ALPHA data format. Do not use it.\n\n");
+
+	if (params->add_comments) {
+		printf ("# %s\n", a->model);
+	}
+
+	printf("# automatically generated camera record %d/%d\n", i+1, total);
+	printf("device \"%s\" {\n", a->model);
+
+	printf("    device_type");
+	first = 1;
+	gpi_enum_to_string(a->device_type, 
+			   gpi_gphoto_device_type_map,
+			   ddb_list_out_func,
+			   (void *) &first);
+	printf(";\n");
+
+	printf("    driver \"%s\";\n", camlib_basename);
+
+	printf("    driver_status");
+	first = 1;
+	gpi_enum_to_string(a->status, 
+			   gpi_camera_driver_status_map,
+			   ddb_list_out_func,
+			   (void *) &first);
+	printf(";\n");
+
+	printf("    operations");
+	first = 1;
+	gpi_flags_to_string_list(a->operations, 
+				 gpi_camera_operation_map,
+				 ddb_list_out_func,
+				 (void *) &first);
+	printf(";\n");
+
+	printf("    file_operations");
+	first = 1;
+	gpi_flags_to_string_list(a->file_operations, 
+				 gpi_file_operation_map,
+				 ddb_list_out_func,
+				 (void *) &first);
+	printf(";\n");
+
+	printf("    folder_operations");
+	first = 1;
+	gpi_flags_to_string_list(a->folder_operations, 
+				 gpi_folder_operation_map,
+				 ddb_list_out_func,
+				 (void *) &first);
+	printf(";\n");
+}
+
+
+static int
+ddb_camera_func (const func_params_t *params, 
+		 const int i,
+		 const int total,
+		 const CameraAbilities *a)
+{
+	const char *camlib_basename = basename(a->library);
+	int head_printed = 0;
+#define DELAYED_HEAD() \
+	do { \
+		if (!head_printed) {			      \
+			ddb_delayed_head(params, i, total,    \
+					 a, camlib_basename); \
+			head_printed = 1;		      \
+		} \
+	} while (0)
+
+	if ((a->port & GP_PORT_SERIAL)) {
+		int first = 1;
+		DELAYED_HEAD();
+		printf("    interface serial {\n");
+		if (a->speed[0] != 0) {
+			unsigned int i;
+			printf("        speeds");
+			for (i=0; 
+			     ((i < (sizeof(a->speed)/sizeof(a->speed[0]))) &&
+			      (a->speed[i] != 0)); i++) {
+				printf("%s %d", (first)?"":",", a->speed[i]);
+				first = 0;
+			}
+			printf(";\n");
+		}
+		printf("    };\n");
+	}
+
+	if ((a->port & GP_PORT_USB)) {
+		if (a->usb_vendor) { 
+			/* usb product id may have the legal value zero! */
+			DELAYED_HEAD();
+			printf("    interface usb {\n");
+			printf("        vendor  0x%04x;\n", a->usb_vendor);
+			printf("        product 0x%04x;\n", a->usb_product);
+			printf("    };\n");
+		}
+		if ((a->usb_class) && (a->usb_class != 666)) {
+			DELAYED_HEAD();
+			printf("    interface usb {\n");
+			printf("        class 0x%02x;\n", a->usb_class);
+			if (a->usb_subclass != -1) {
+				printf("        subclass 0x%02x;\n", a->usb_subclass);
+			}
+			if (a->usb_protocol != -1) {
+				printf("        protocol 0x%02x;\n", a->usb_protocol);
+			}
+			printf("    };\n");
+		}
+	}
+	if ((a->port & GP_PORT_DISK)) {
+		DELAYED_HEAD();
+		printf("    interface disk;\n");
+	}
+	if ((a->port & GP_PORT_PTPIP)) {
+		DELAYED_HEAD();
+		printf("    interface ptpip;\n");
+	}
+#undef DELAYED_HEAD
+
+	if (head_printed) {
+		printf("    # driver_options {\n");
+		printf("    #      option \"foo\";\n");
+		printf("    #      option \"bar\" \"bla\";\n");
+		printf("    # };\n");
+
+		printf("}; # %s\n\n", a->model);
+	}
+	return 0;
+}
+
+
 /* print_fdi_map
  *
  * Print FDI Device Information file for HAL with information on 
@@ -378,6 +728,7 @@ fdi_begin_func (const func_params_t *params)
 static int
 fdi_camera_func (const func_params_t *params, 
 		 const int i,
+		 const int total,
 		 const CameraAbilities *a)
 {
 	char	*s, *d, model[256];
@@ -398,51 +749,65 @@ fdi_camera_func (const func_params_t *params,
 	}
 	*d = '\0';
 
-	if (a->usb_vendor == 0x07b4 && a->usb_product == 0x105) {
-		/* Marcus says: The Olympus Sierra/Storage dual mode camera firmware.
-		 * Some HAL using software gets deeply confused by this being here
-		 * and also detected as mass storage elsewhere, so blacklist
-		 * it here.
-		 */
-		return 0;
-	}
-	if (a->usb_vendor) { /* usb product id might be 0! */
-		printf("   <match key=\"usb.vendor_id\" int=\"%d\">\n", a->usb_vendor);
-		printf("    <match key=\"usb.product_id\" int=\"%d\">\n", a->usb_product);
-		printf("     <merge key=\"info.category\" type=\"string\">camera</merge>\n");
-		printf("     <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
-
-		/* HACK alert ... but the HAL / gnome-volume-manager guys want that */
-		if (NULL!=strstr(a->library,"ptp"))
-			printf("     <merge key=\"camera.access_method\" type=\"string\">ptp</merge>\n");
-		else
-			printf("     <merge key=\"camera.access_method\" type=\"string\">proprietary</merge>\n");
-		printf("     <merge key=\"camera.libgphoto2.name\" type=\"string\">%s</merge>\n", model);
-		printf("     <merge key=\"camera.libgphoto2.support\" type=\"bool\">true</merge>\n");
-		printf("    </match>\n");
-		printf("   </match>\n");
-
-	} else if (a->usb_class) {
-		printf("   <match key=\"usb.interface.class\" int=\"%d\">\n", a->usb_class);
-		printf("    <match key=\"usb.interface.subclass\" int=\"%d\">\n", a->usb_subclass);
-		printf("     <match key=\"usb.interface.protocol\" int=\"%d\">\n", a->usb_protocol);
-		printf("      <merge key=\"info.category\" type=\"string\">camera</merge>\n");
-		printf("      <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
-		if (a->usb_class == 6) {
-			printf("      <merge key=\"camera.access_method\" type=\"string\">ptp</merge>\n");
-		} else {
-			if (a->usb_class == 8) {
-				printf("      <merge key=\"camera.access_method\" type=\"string\">storage</merge>\n");
-			} else {
-				printf("      <merge key=\"camera.access_method\" type=\"string\">proprietary</merge>\n");
-			}
+	if ((a->port & GP_PORT_USB)) {
+		if (a->usb_vendor == 0x07b4 && a->usb_product == 0x105) {
+			/* Marcus says: The Olympus Sierra/Storage dual mode camera firmware.
+			 * Some HAL using software gets deeply confused by this being here
+			 * and also detected as mass storage elsewhere, so blacklist
+			 * it here.
+			 */
+			return 0;
 		}
-		printf("      <merge key=\"camera.libgphoto2.name\" type=\"string\">%s</merge>\n", model);
-		printf("      <merge key=\"camera.libgphoto2.support\" type=\"bool\">true</merge>\n");
-		printf("     </match>\n");
-		printf("    </match>\n");
-		printf("   </match>\n");
-	}
+		if (a->usb_vendor) { /* usb product id might be 0! */
+			printf("   <match key=\"usb.vendor_id\" int=\"%d\">\n", a->usb_vendor);
+			printf("    <match key=\"usb.product_id\" int=\"%d\">\n", a->usb_product);
+			printf("     <merge key=\"info.category\" type=\"string\">camera</merge>\n");
+			if (a->device_type & GP_DEVICE_AUDIO_PLAYER) {
+				printf("     <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
+				printf("     <merge key=\"info.category\" type=\"string\">portable_audio_player</merge>\n");
+				printf("     <append key=\"info.capabilities\" type=\"strlist\">portable_audio_player</append>\n");
+				printf("     <merge key=\"portable_audio_player.access_method\" type=\"string\">user</merge>\n");
+				printf("     <merge key=\"portable_audio_player.type\" type=\"string\">mtp</merge>\n");
+				
+				/* FIXME: needs true formats ... But all of them can do MP3 */
+				printf("     <append key=\"portable_audio_player.output_formats\" type=\"strlist\">audio/mpeg</append>\n");
+			} else {
+				printf("     <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
+				
+				/* HACK alert ... but the HAL / gnome-volume-manager guys want that */
+				if (NULL!=strstr(a->library,"ptp"))
+					printf("     <merge key=\"camera.access_method\" type=\"string\">ptp</merge>\n");
+				else
+					printf("     <merge key=\"camera.access_method\" type=\"string\">proprietary</merge>\n");
+			}
+			/* leave them here even for audio players */
+			printf("     <merge key=\"camera.libgphoto2.name\" type=\"string\">%s</merge>\n", model);
+			printf("     <merge key=\"camera.libgphoto2.support\" type=\"bool\">true</merge>\n");
+			printf("    </match>\n");
+			printf("   </match>\n");
+			
+		} else if ((a->usb_class) && (a->usb_class != 666)) {
+			printf("   <match key=\"usb.interface.class\" int=\"%d\">\n", a->usb_class);
+			printf("    <match key=\"usb.interface.subclass\" int=\"%d\">\n", a->usb_subclass);
+			printf("     <match key=\"usb.interface.protocol\" int=\"%d\">\n", a->usb_protocol);
+			printf("      <merge key=\"info.category\" type=\"string\">camera</merge>\n");
+			printf("      <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
+			if (a->usb_class == 6) {
+				printf("      <merge key=\"camera.access_method\" type=\"string\">ptp</merge>\n");
+			} else {
+				if (a->usb_class == 8) {
+					printf("      <merge key=\"camera.access_method\" type=\"string\">storage</merge>\n");
+				} else {
+					printf("      <merge key=\"camera.access_method\" type=\"string\">proprietary</merge>\n");
+				}
+			}
+			printf("      <merge key=\"camera.libgphoto2.name\" type=\"string\">%s</merge>\n", model);
+			printf("      <merge key=\"camera.libgphoto2.support\" type=\"bool\">true</merge>\n");
+			printf("     </match>\n");
+			printf("    </match>\n");
+			printf("   </match>\n");
+		}
+	} /* camera has USB connection */
 	return 0;
 }
 
@@ -481,6 +846,7 @@ fdi_device_begin_func (const func_params_t *params)
 static int
 fdi_device_camera_func (const func_params_t *params, 
 		 	const int i,
+			const int total,
 		 	const CameraAbilities *a)
 {
 	char	*s, *d, model[256];
@@ -500,36 +866,43 @@ fdi_device_camera_func (const func_params_t *params,
 		s++;
 	}
 	*d = '\0';
-	if (a->usb_vendor == 0x07b4 && a->usb_product == 0x105) {
-		/* Marcus says: The Olympus Sierra/Storage dual mode camera firmware.
-		 * Some HAL using software gets deeply confused by this being here
-		 * and also detected as mass storage elsewhere, so blacklist
-		 * it here.
-		 */
-		return 0;
-	}
-	if (a->usb_vendor) { /* usb product id might be 0! */
-		/* do not set category. We don't really know what this device really is.
-		 * But we do now that is capable of being a camera, so add to capabilities
-		 */
-		printf("   <match key=\"usb_device.vendor_id\" int=\"%d\">\n", a->usb_vendor);
-		printf("    <match key=\"usb_device.product_id\" int=\"%d\">\n", a->usb_product);
-		printf("     <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
-		printf("    </match>\n");
-		printf("   </match>\n");
-	}
+
+	if ((a->port & GP_PORT_USB)) {
+
+		if (a->usb_vendor == 0x07b4 && a->usb_product == 0x105) {
+			/* Marcus says: The Olympus Sierra/Storage dual mode camera firmware.
+			 * Some HAL using software gets deeply confused by this being here
+			 * and also detected as mass storage elsewhere, so blacklist
+			 * it here.
+			 */
+			return 0;
+		}
+		if (a->usb_vendor) { /* usb product id might be 0! */
+			/* do not set category. We don't really know what this device really is.
+			 * But we do now that is capable of being a camera, so add to capabilities
+			 */
+			printf("   <match key=\"usb_device.vendor_id\" int=\"%d\">\n", a->usb_vendor);
+			printf("    <match key=\"usb_device.product_id\" int=\"%d\">\n", a->usb_product);
+			if (a->device_type & GP_DEVICE_AUDIO_PLAYER)
+				printf("     <append key=\"info.capabilities\" type=\"strlist\">portable_audio_player</append>\n");
+			else
+				printf("     <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
+			printf("    </match>\n");
+			printf("   </match>\n");
+		}
 #if 0
-	/* would need to be able to merge upwards ... but cannot currently */
-	else if (a->usb_class) {
-		printf("   <match key=\"usb.interface.class\" int=\"%d\">\n", a->usb_class);
-		printf("    <match key=\"usb.interface.subclass\" int=\"%d\">\n", a->usb_subclass);
-		printf("     <match key=\"usb.interface.protocol\" int=\"%d\">\n", a->usb_protocol);
-		printf("      <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
-		printf("     </match>\n");
-		printf("    </match>\n");
-		printf("   </match>\n");
-	}
+		/* would need to be able to merge upwards ... but cannot currently */
+		else if ((a->usb_class) && (a->usb_class != 666)) {
+			printf("   <match key=\"usb.interface.class\" int=\"%d\">\n", a->usb_class);
+			printf("    <match key=\"usb.interface.subclass\" int=\"%d\">\n", a->usb_subclass);
+			printf("     <match key=\"usb.interface.protocol\" int=\"%d\">\n", a->usb_protocol);
+			printf("      <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
+			printf("     </match>\n");
+			printf("    </match>\n");
+			printf("   </match>\n");
+		}
 #endif
+	}
 	return 0;
 }
 
@@ -575,7 +948,7 @@ typedef struct {
 static int
 iterate_camera_list (const int add_comments, 
 		     const output_format_t *format, 
-		     char *argv[])
+		     string_array_p argv)
 {
 	int number_of_cameras;
 	CameraAbilitiesList *al;
@@ -585,11 +958,11 @@ iterate_camera_list (const int add_comments,
 	CR (gp_abilities_list_new (&al));
 	CR (gp_abilities_list_load (al, NULL)); /* use NULL context */
 	CR (number_of_cameras = gp_abilities_list_count (al));
-	
+
 	params.add_comments = add_comments;
 	params.number_of_cameras = number_of_cameras;
-	params.argv = &argv;
-	
+	params.argv = argv;
+
 	if (format->begin_func != NULL) {
 		format->begin_func(&params);
 	}
@@ -598,7 +971,7 @@ iterate_camera_list (const int add_comments,
 		int i;
 		for (i = 0; i < number_of_cameras; i++) {
 			CR (gp_abilities_list_get_abilities (al, i, &a));
-			format->camera_func(&params, i, &a);
+			format->camera_func(&params, i, number_of_cameras, &a);
 		}
 	}
 
@@ -648,13 +1021,21 @@ static const output_format_t formats[] = {
 	 end_func: fdi_device_end_func
 	},
 	{name: "udev-rules",
-	 descr: "udev rules file",
-	 help: "Put it into /etc/udev/libgphoto2.rules",
-	 paramdescr: "<PATH_TO_HOTPLUG_SCRIPT>",
+	 descr: "udev < 0.98 rules file",
+	 help: "Put it into /etc/udev/libgphoto2.rules, set file mode, owner, group or add script to run",
+	 paramdescr: "( <PATH_TO_SCRIPT> | [mode <mode>|owner <owner>|group <group>]* ) ",
 	 begin_func: udev_begin_func, 
 	 camera_func: udev_camera_func,
 	 end_func: udev_end_func
 	},
+	{name: "udev-rules-0.98",
+	 descr: "udev >= 0.98 rules file",
+	 help: "Put it into /etc/udev/libgphoto2.rules, set file mode, owner, group or add script to run, for udev >= 0.98",
+	 paramdescr: "( <PATH_TO_SCRIPT> | [mode <mode>|owner <owner>|group <group>]* ) ",
+	 begin_func: udev_098_begin_func, 
+	 camera_func: udev_098_camera_func,
+	 end_func: udev_end_func
+	},
 	{name: "idlist",
 	 descr: "list of IDs and names",
 	 help: "grep for an ID to find the device name",
@@ -663,6 +1044,14 @@ static const output_format_t formats[] = {
 	 camera_func: idlist_camera_func,
 	 end_func: empty_end_func
 	},
+	{name: "gp2ddb",
+	 descr: "gphoto2 device database (PRE-ALPHA)",
+	 help: "PRE-ALPHA test stage, do not use for production! Machine parseable.",
+	 paramdescr: NULL,
+	 begin_func: ddb_begin_func,
+	 camera_func: ddb_camera_func,
+	 end_func: ddb_end_func
+	},
 	{NULL, NULL, NULL, NULL, 
 	 NULL, NULL, NULL}
 };
@@ -713,13 +1102,14 @@ int main(int argc, char *argv[])
 
 	char *format_name = NULL; /* name of desired output format */
 	int format_index;         /* index number of (desired) output format */
-	static char *fmt_argv[8];
+	static char *fmt_argv[16]; /* format specific arguments */
 
 	int i, j;
+	unsigned int ui;
 
 	/* initialize parameters to NULL */
-	for (i=0; i<(sizeof(fmt_argv)/sizeof(fmt_argv[0])); i++) {
-		fmt_argv[i] = NULL;
+	for (ui=0; ui<(sizeof(fmt_argv)/sizeof(fmt_argv[0])); ui++) {
+		fmt_argv[ui] = NULL;
 	}
 
 	/* walk through command line arguments until format is encountered */
@@ -763,13 +1153,15 @@ int main(int argc, char *argv[])
 	}
 
 	/* copy remaining arguments */
-	for (j=i; (j<argc) && ((j-i)<((sizeof(fmt_argv)/sizeof(fmt_argv[0]))-1)); j++) {
+	for (j=i; 
+	     (j<argc) && ((j-i)<(int)((sizeof(fmt_argv)/sizeof(fmt_argv[0]))-1)); 
+	     j++) {
 		fmt_argv[j-i] = argv[j];
 	}
 
 	/* execute the work using the given parameters*/
 	return iterate_camera_list(add_comments, &formats[format_index],
-				   fmt_argv
+				   &fmt_argv
 		);
 }
 
