@@ -1,4 +1,4 @@
-/* $Id: print-usb-usermap.c 8520 2006-01-23 18:20:35Z hun $
+/* $Id: print-usb-usermap.c 9221 2006-09-28 03:04:21Z hun $
  *
  * Copyright  2002 Hans Ulrich Niedermann <hun@users.sourceforge.net>
  * Portions Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
@@ -54,8 +54,8 @@ ARGV0 " prints the lines for the usb.usermap file on stdout.\n" \
 #include <time.h>
 #include <string.h>
 
-#include <gphoto2-camera.h>
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-camera.h>
+#include <gphoto2/gphoto2-port-log.h>
 
 #ifndef TRUE
 #define TRUE  (0==0)
